/*
**  ActionController.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _Musicbox_MusicBox_H_ActionController
#define _Musicbox_MusicBox_H_ActionController

#include <Foundation/NSObject.h>
#include <Mixer/Mixer.h>
#include <MusicBox/EncoderModule.h>

@class SongInfo;
@class PlaylistManager;
@class LED;
@class NSTimer;
@class NSNumber;
@class NSMutableDictionary;
@class NSSound;

typedef enum _ActionControllerState {
  ActionControllerPlaying = 1,
  ActionControllerPaused = 2,
  ActionControllerStopping = 4, /* [NSSound stop] */
  ActionControllerStopped = 8 /* didFinishPlaying in delegate is called */
} ActionControllerState;

@interface ActionController: NSObject
{
  PlaylistManager *playlistManager;
  Mixer *mixer;

  ActionControllerState currentState;

  /* For playing */
  int nowPlayTrack, nextPlayTrack;
  SongInfo *playingSongInfo;
  BOOL willContinue; /* Continue play after song finish */
  BOOL isRepeat; /* Repeat the play list */
  BOOL isShuffle;

  /* For encoding */
/*
  id <EncoderModule> encoder;
  int nowEncodeTrack;
  BOOL isEncoding;
  BOOL isWaitingPlayingStopped;
*/

  /* For both playing and encoding */
  LED *led;
  NSTimer *timer;
}

+ (ActionController *) sharedController;
- (void) setLED: (LED *) led;

/* There methods are triggerred by buttons.
 * They decide the song to play, and call _playSong method to play.
 * Each method has to consider every possible situation itself.
 * playSong only play the song of nowPlayTrack variable.
 */
- (void) play: (id)sender;
- (void) prev: (id)sender;
- (void) next: (id)sender;
- (void) stop: (id)sender;
- (void) pause: (id)sender;
- (void) seek: (NSNumber *) seconds by: (id)sender;

- (ActionControllerState) currentState;
- (SongInfo *) playingSongInfo;

/* The general encoding method */
//- (void) encodeSong: (int) track;

- (void) updateLED: (id) sender;

// Called by volume
- (void) sliderAction: (id) sender;
- (float) volume;

// Called by double click of play list view
- (void) doubleClickAction: (id) sender;

// Called by MusicBoxWindow or menu
- (void) setShuffle: (id) sender;
- (void) setRepeat: (id) sender;

// Delegate method for NSSound
- (void) sound: (NSSound *) aSound didFinishPlaying: (BOOL) _bool;

/* Called by encoder */
//- (void) encoder: (id <EncoderModule>) encoder didFinishEncoding: (BOOL) bool;

@end

@interface ActionController (Private)

/* play the songInfoForPlay in PlaylistManager */
- (void) _playSong;
/* Search for next/prev song */
- (int) _nextSong;
- (int) _prevSong;

@end

#endif /* _Musicbox_MusicBox_H_ActionController */
