/*
**  AudioCD.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioCD.h"
#include <MusicBox/ModuleLoader.h>
#include <ShengGuang/CDSound.h>
#include <ShengGuang/SGSystem.h>
#include <ShengGuang/SongInfo.h>
#include <Cddb/Cddb.h>
#include "CddbWindowController.h"
#include <Foundation/Foundation.h>
#include <AppKit/NSButton.h>
#include <AppKit/NSApplication.h>
#include <AppKit/NSImage.h>

@implementation AudioCD

+ (NSString *) classType
{
  return @"Audio CD";
}

- (void) readFromCddb: (id) sender
{
  int i = 0, numtracks = 0;
  SongInfo *info;
  NSArray *trackArray;

  NSLog(@"Read from cddb");

  // Not interfere when CD is playing 
/*
  if([[SGSystem sharedSystem] isCDPlaying])
    {
      return;
    }
*/

  trackArray = [[SGSystem sharedSystem] allCDTracks];

  if ([trackArray count] > 0)
    {
      Cddb *cddb = [cddbClass new];
      NSDictionary *cddbEntry = nil;
      id object;

      RETAIN(trackArray);

      numtracks = [trackArray count];

      object = [[NSUserDefaults standardUserDefaults] objectForKey: @"FreedbSite"];
      if ( object )  
        {
          [cddb setDefaultSite: object];

          if([cddb connect])
            {
              int userSelection;
              NSArray *queryArray = [cddb queryWithCDTracks: trackArray];

              if ([queryArray count] == 1)
                {
                  // One match
                  userSelection = 0;
                }
              else if ([queryArray count] > 1) // Multiple Matches
                { 
                  CddbWindowController *cddbController;
                  NSAutoreleasePool *pool;
                  int result;
 
                  pool = [[NSAutoreleasePool alloc] init];
             
                  cddbController = [[CddbWindowController alloc] initWithWindowNibName: @"CddbWindow"];
                  [cddbController setDataSource: queryArray];
                  result = [NSApp runModalForWindow: [cddbController window]];

                  if (result == NSRunStoppedResponse)
                    {
                      userSelection = [cddbController userSelection];
                    }
                  else
                    {
                      userSelection = 0;
                    }
            
                  RELEASE(cddbController);
                  RELEASE(pool);
                }
              else
                {
                  // No Match
                  [cddb disconnect];
                  RELEASE(cddb);
                  return;
                }
              cddbEntry = [cddb readWithCategory: [[queryArray objectAtIndex: userSelection] objectForKey: @"category"]
                                          discid: [[queryArray objectAtIndex: userSelection] objectForKey: @"discid"]
                                     postProcess: YES];
              RETAIN(cddbEntry);
              [cddb disconnect];
            }
        }

      RELEASE(cddb);

      i = 0;
      while (i < numtracks)
        {
          info = [listArray objectAtIndex: i];
          if ([[cddbEntry objectForKey: @"titles"] objectAtIndex: i]) {
            [info setSong: [[cddbEntry objectForKey: @"titles"] objectAtIndex: i]];
          }

          if ([[cddbEntry objectForKey: @"artists"] objectAtIndex: i]) {
            [info setArtist: [[cddbEntry objectForKey: @"artists"] objectAtIndex: i]];
          }

          if ([cddbEntry objectForKey: @"album"]) {
            [info setAlbum: [cddbEntry objectForKey: @"album"]];
          }

          i++;
        }
      RELEASE(cddbEntry);
      RELEASE(trackArray);
    }

  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBMusicBoxWindowNeedReloadNotification"
           object: nil];
}

- (void) ejectCD: (id) sender
{
  if ([[SGSystem sharedSystem] ejectCD])
    {
      [listArray removeAllObjects];
      [discid setString: nil];

      [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBShouldReloadPlaylistNotification"
           object: nil];

      [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBPlaylistSelectionChangedNotification"
           object: self];
    }
}

- (NSButton *) additionalButtonOnPlaylistWindow
{
  return buttonOnPlaylist;
}

- (NSButton *) additionalButtonOnMusicBoxWindow
{
  return buttonOnMusicBox;
}

- (NSArray *) soundList
{
  int i = 0, numtracks = 0;
  CDSound *tempSound;
  SongInfo *info;
  NSArray *trackArray;

  /* Not interfere when CD is playing */
  if([[SGSystem sharedSystem] isCDPlaying])
    {
      return listArray;
    } 
  
  NSLog(@"Read CD Content");
  trackArray = [[SGSystem sharedSystem] allCDTracks];

  if ([trackArray count] > 0)
    {
      NSString *newDiscid;
      Cddb *cddb = [cddbClass new];

      RETAIN(trackArray);

      numtracks = [trackArray count];

      /* newDiscid is autoreleased */
      newDiscid = [cddb discidWithCDTracks: trackArray
                                   locally: YES];

      RETAIN(newDiscid);
   
      if ([newDiscid isEqualToString: discid]) 
        {
          RELEASE(trackArray);
          RELEASE(cddb);
          RELEASE(newDiscid);
          return listArray;
        }

      /* New CD */
      /* FIXME: The memory about discid is weird (retainCount) */
      [discid setString: newDiscid]; 
      [listArray removeAllObjects];

      i = 0;
      while (i < numtracks)
        {
          tempSound = [CDSound new];
          [tempSound setTrack: i];

          info = [SongInfo new];
          [info setSong: [NSString stringWithFormat: @"Track %d", i+1]];
          [info setTrack: [NSNumber numberWithInt: i+1]];
          [info setStatus: [NSNumber numberWithInt: i+1]];
          [info setChecked: YES];
          [info setDuration: [NSNumber numberWithInt: ([[[trackArray objectAtIndex: i] objectForKey: @"length"] intValue] / CD_FPS)]];
          [info setSound: tempSound];
          [listArray addObject: info];
          RELEASE(tempSound);
          RELEASE(info);
          i++;
        }
      RELEASE(cddb);
      RELEASE(newDiscid);
      RELEASE(trackArray);
    }
  else 
    {
      [listArray removeAllObjects];
      trackArray = nil;
    }
  return listArray;
}

- (id)initWithOwner: (id)owner
{
  ModuleLoader *loader = [ModuleLoader defaultLoader];

  self = [super initWithOwner: owner];
  [self setIcon: [NSImage imageNamed: @"stock_cdrom_16.png"]];
  [self setName: @"Audio CD"];
  discid = [[NSMutableString alloc] initWithString: nil];

  buttonOnPlaylist = [[NSButton alloc] initWithFrame: NSMakeRect(0, 0, 30, 30)];
  [buttonOnPlaylist setButtonType: NSMomentaryPushButton];
//  [buttonOnPlaylist setImagePosition: NSImageOnly];
//  [buttonOnPlaylist setImage: [NSImage imageNamed: @"stock_add_24.png"]];
  [buttonOnPlaylist setTitle: @"eject"];
  [buttonOnPlaylist setAutoresizingMask: NSViewMinXMargin | NSViewMinYMargin];
  [buttonOnPlaylist setTarget: self];
  [buttonOnPlaylist setAction: @selector(ejectCD:)];

  buttonOnMusicBox = [[NSButton alloc] initWithFrame: NSMakeRect(0, 0, 30, 30)];
  [buttonOnMusicBox setButtonType: NSMomentaryPushButton];
//  [buttonOnMusicBox setImagePosition: NSImageOnly];
//  [buttonOnMusicBox setImage: [NSImage imageNamed: @"stock_add_24.png"]];
  [buttonOnMusicBox setTitle: @"cddb"];
  [buttonOnMusicBox setAutoresizingMask: NSViewMinXMargin | NSViewMinYMargin];
  [buttonOnMusicBox setTarget: self];
  [buttonOnMusicBox setAction: @selector(readFromCddb:)];

  cddbClass = [loader classNameOf: @"Cddb.bundle"];

  if (!cddbClass)
    {
      NSLog(@"Failed to get Class from Cddb.bundle");
    }

  return self;
}
 
- (NSArray *) dataForSave
{
  return nil;
}

- (void) dealloc
{
  RELEASE(discid);
  RELEASE(buttonOnPlaylist);
  RELEASE(buttonOnMusicBox);
  [super dealloc];
}

- (BOOL) canDragSoundList
{
  /* It should be YES after import implemented */
  return NO;
}

- (BOOL) canDropSoundList
{
  return NO;
}

- (BOOL) loadData: (NSArray *)data
{
  return NO;
}

- (BOOL) addObject: (SongInfo *) aSongInfo
{
  return NO;
}

- (BOOL) addObjects: (NSArray *) aArray
{
  return NO;
}

- (BOOL) deleteObject: (id) object
{
  return NO;
}

- (BOOL) insertObject: (id) object atIndex: (int) index
{
  return NO;
}

@end
