/*
**  CddbWindow.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "CddbWindow.h"
#include <AppKit/AppKit.h>

@implementation CddbWindow

- (NSTableView *) tableView
{
  return tableView;
}

- (void) dealloc
{
  [super dealloc];
}

- (void) layoutWindow
{
  NSRect frame, bounds;
  NSTextField *desc;
  NSTableColumn *categoryCol, *descCol;
  NSScrollView *scrollView;  

  bounds = [[self contentView] bounds];

  frame = NSMakeRect(5, bounds.size.height-25, 240, 22);
  desc = [[NSTextField alloc] initWithFrame: frame];
  [desc setStringValue: _(@"Pick one:")];
  [desc setEditable: NO];
  [desc setSelectable: NO];
  [desc setBordered: NO];
  [desc setBezeled: NO];
  [desc setBackgroundColor: [NSColor controlBackgroundColor]];
  [desc setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: desc];
  RELEASE(desc);

  categoryCol = [[NSTableColumn alloc] initWithIdentifier: @"category"];
  [categoryCol setEditable: NO];
  [categoryCol setMaxWidth: 80];
  [[categoryCol headerCell] setStringValue: _(@"Category")];

  descCol = [[NSTableColumn alloc] initWithIdentifier: @"description"];
  [descCol setEditable: NO];
  [[descCol headerCell] setStringValue: _(@"Description")];

  frame = NSMakeRect(0,30,bounds.size.width, (bounds.size.height-60));
  scrollView = [[NSScrollView alloc] initWithFrame: frame];
  [scrollView setHasHorizontalScroller:YES];
  [scrollView setHasVerticalScroller:YES];
  [scrollView setBorderType: NSBezelBorder];
  [scrollView setAutoresizingMask: (NSViewWidthSizable 
                    | NSViewHeightSizable)];

  frame = NSMakeRect(0,30,bounds.size.width,(bounds.size.height-60));
  tableView = [[NSTableView alloc] initWithFrame: frame];
  [tableView setDrawsGrid: NO];
  [tableView addTableColumn: categoryCol];
  [tableView addTableColumn: descCol];
  [tableView setAutoresizesAllColumnsToFit: YES];
  [tableView setAllowsMultipleSelection: NO];
  [tableView setAllowsEmptySelection: NO];
  [tableView setTarget: [self windowController]];
  [tableView setDataSource: [self delegate]];
  [tableView setDelegate: [self delegate]];
  [tableView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [tableView setDoubleAction: @selector(doubleClickOnTable:)];

  [scrollView setDocumentView: tableView];
  [[self contentView] addSubview: scrollView];

  RELEASE(categoryCol);
  RELEASE(descCol);
  RELEASE(scrollView);
  RELEASE(tableView);

  //add ok/cancel buttons
  frame = NSMakeRect((bounds.size.width-130), 5, 60, 22);
  cancelButton = [[NSButton alloc] initWithFrame: frame];
  [cancelButton setButtonType: NSMomentaryPushButton];
  [cancelButton setTitle: _(@"Cancel")];
  [cancelButton setTarget: [self windowController]];
  [cancelButton setAction: @selector(cancelClicked:)];
  [cancelButton setAutoresizingMask: NSViewMinXMargin];
  [[self contentView] addSubview: cancelButton];
  RELEASE(cancelButton);

  frame = NSMakeRect((bounds.size.width-65), 5, 60, 22);
  okButton = [[NSButton alloc] initWithFrame: frame];
  [okButton setButtonType:NSMomentaryPushButton];
  [okButton setKeyEquivalent: @"\r"];
  [okButton setImagePosition: NSImageRight];
  [okButton setImage: [NSImage imageNamed: @"common_ret"]];
  [okButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];
  [okButton setTitle: _(@"OK")];
  [okButton setTarget: [self windowController]];
  [okButton setAction: @selector(okClicked:)];
  [okButton setAutoresizingMask: NSViewMinXMargin];
  [[self contentView] addSubview:okButton];
  RELEASE(okButton);
}

@end
