/*
**  CddbWindowController.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "CddbWindowController.h"
#include "CddbWindow.h"
#include <AppKit/AppKit.h>

@implementation CddbWindowController

- (int) numberOfRowsInTableView: (NSTableView *) view
{
  if (dataSource)
    return [dataSource count];
  else
    return 0;
}

- (id) tableView: (NSTableView *) view
                  objectValueForTableColumn: (NSTableColumn *) column
                  row: (int) row
{
  return [[dataSource objectAtIndex: row] objectForKey: [column identifier]];
}

- (void) setDataSource: (NSArray *) array
{
  ASSIGN(dataSource, array);
  [tableView reloadData];
  if ([dataSource count] > 0)
    [tableView selectRow: 0
               byExtendingSelection: NO];
}

- (int) userSelection
{
  return userSelection;
}

- (id) initWithWindowNibName: (NSString *) windowNibName
{
  CddbWindow *window;
  
  window = [[CddbWindow alloc] initWithContentRect: NSMakeRect(300,300,500,200)
					 styleMask: ( NSTitledWindowMask |
                                                      NSResizableWindowMask )
					 backing: NSBackingStoreBuffered
					 defer: YES];
  
  self = [super initWithWindow: window];
  
  [window setDelegate: self];
  [window layoutWindow];

  tableView = [window tableView];

  RELEASE(window);

  // We set our window title
  [[self window] setTitle: _(@"Cddb Chooser")];

  return self;
}


- (void) dealloc
{
  [super dealloc];
}

// action methods

- (void) doubleClickOnTable: (id) sender
{
  [self okClicked: sender];
}

- (void) okClicked: (id) sender
{  
  userSelection = [tableView selectedRow];
  [NSApp stopModal];
  [self close];
}

- (void) cancelClicked: (id) sender
{
  [NSApp abortModal];
  [self close];
}

@end
