/*
**  AudioLibrary.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioLibrary.h"
#include <ShengGuang/SongInfo.h>
#include <ShengGuang/SGSystem.h>
#include <ShengGuang/OGGEditor.h>
#include <AppKit/AppKit.h>

@implementation AudioLibraryView
- (id) initWithFrame: (NSRect) frame
{
  NSTextField *pathLabel;

  self = [super initWithFrame: frame];

  pathLabel = [[NSTextField alloc] initWithFrame: NSMakeRect(0, 145, 70, 20)];
  [pathLabel setStringValue: _(@"Source Path:")];
  [pathLabel setEditable: NO];
  [pathLabel setSelectable: NO];
  [pathLabel setBordered: NO];
  [pathLabel setBezeled: NO];
  [pathLabel setBackgroundColor: [NSColor controlBackgroundColor]];
  [self addSubview: pathLabel];
  RELEASE(pathLabel);

  pathField = [[NSTextField alloc] initWithFrame: NSMakeRect(75, 145, 160, 20)];
  [pathField setDelegate: [_window windowController]];
  [pathField setTarget: [_window windowController]];
  [pathField setAction: @selector(okClicked:)];
  [self addSubview: pathField];
  RELEASE(pathField);

  return self;
}

- (NSDictionary *) property
{
  return [NSDictionary dictionaryWithObjectsAndKeys: 
                         [nameField stringValue], @"name",
                         [pathField stringValue], @"path",
                         nil];
}

- (void) setProperty: (NSDictionary *) property
{
  [nameField setStringValue: [property objectForKey: @"name"]];
  [pathField setStringValue: [property objectForKey: @"path"]];
}

@end

@implementation AudioLibrary

+ (NSString *) classType
{
  return @"Library";
}

+ (AudioListView *) settingView
{
  return AUTORELEASE([[AudioLibraryView alloc] initWithFrame: NSMakeRect(5, 50, 240, 180)]);
}

- (BOOL) canDragSoundList
{
  return YES;
}

- (BOOL) canDropSoundList
{
  return NO;
}

- (BOOL) deleteObject: (id) object
{
  return NO;
}

- (BOOL) insertObject: (id) object atIndex: (int) index
{
  return NO;
}

- (void) setProperty: (NSDictionary *) dict
{
  if ([dict objectForKey: @"name"])
    [self setName: [dict objectForKey: @"name"]];

  ASSIGN(path, [[dict objectForKey: @"path"] stringByStandardizingPath]);

  /* Reload song list after path changed */
  [self loadData: [self dataForSave]];
}

- (NSDictionary *) property
{
  if (path)
    return [NSDictionary dictionaryWithObject: path forKey: @"path"];
  else
    return nil;
}

- (id)initWithOwner: (id)owner
{
  self = [super initWithOwner: owner];
  manager = [NSFileManager defaultManager];
  [self setName: @"Library"];
  [self setIcon: [NSImage imageNamed: @"stock_home_16.png"]];
  return self;
}

- (void)dealloc
{
  RELEASE(path);
  [super dealloc];
} 

/* In AudioFile, it has to check the audio format of each file 
 * whether SDL support, and get the information from saved information.
 * If it is a new file and no saved information,
 * set every field as "Unknown" and duration is 0
 */
- (BOOL) loadData: (NSArray *) data
{
  NSArray *fileArray, *extArray;
  NSString *eachPath, *ext;
  SongInfo *info;
  int i;
  unsigned int index;
  NSMutableArray *dataArray = [NSMutableArray new];
  id object;

  RETAIN(data);

  [self removeAllObjects];

  /* Get all the file name */ 
  for (i = 0; i < [data count]; i++)
    {
      [dataArray addObject: [[data objectAtIndex: i] objectForKey: @"filename"]];
    }

  /* Search all the supported audio format */
  extArray = [[SGSystem sharedSystem] availableFormats];
  fileArray = [[manager enumeratorAtPath: path] allObjects];

  for (i = 0; i < [fileArray count]; i++)
    {
      eachPath = [path stringByAppendingPathComponent: [fileArray objectAtIndex: i]]; 
      ext = [[eachPath pathExtension] lowercaseString];
      if ([extArray containsObject: ext]) {
        info = [SongInfo new];
        if (((index = [dataArray indexOfObject: eachPath]) != NSNotFound))
          {
            object = [data objectAtIndex: index];
            [info setSong: [object objectForKey: @"song"]];
            [info setArtist: [object objectForKey: @"artist"]];
            [info setAlbum: [object objectForKey: @"album"]];
            [info setFilename: [object objectForKey: @"filename"]];
            [info setDuration: [object objectForKey: @"duration"]];
          }
        else 
          {
            if ([ext isEqualToString: @"ogg"]) 
              {
                OGGEditor *editor = [[OGGEditor alloc] initWithFile: eachPath];
                [info setSong: [editor title]];
                [info setArtist: [editor artist]];
                [info setAlbum: [editor album]];
                [info setDuration: [editor seconds]];
                RELEASE(editor);
              }
            else
              {
                [info setSong: [fileArray objectAtIndex: i]];
              }
            [info setFilename: eachPath];
          }
        [info setStatus: @""];
        [info setChecked: YES];
        [self addObject: info];
        AUTORELEASE(info);
      }
   }
  AUTORELEASE(data);
  return YES;
}

@end
