/*
**  AudioSmart.m
**
**  Copyright (c) 2002
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioSmart.h"
#include <MusicBox/ExtendedTableView.h>
#include <ShengGuang/SongInfo.h>
#include <PlaylistManager.h>
#include <AppKit/AppKit.h>

@implementation AudioSmartView

- (void) addRule: (id) sender
{
  NSMutableDictionary *rule = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                 [categoryButton stringValue], @"category",
                                 [compareButton stringValue], @"compare",
                                 [expressionField stringValue], @"expression",
                                 @"YES", @"state",
                                 nil]; 
  [rules addObject: rule];
  [tableView reloadData];
  [tableView selectRow: ([rules count] - 1) byExtendingSelection: NO];
}

- (void) deleteRule: (id) sender
{
  int selected = [tableView selectedRow];
  if (selected < 0)
    return;
  [rules removeObjectAtIndex: selected];
  [tableView reloadData];
  if ([rules count] < 1)
    {
      [expressionField setStringValue: @""];
      return;
    }
  if (selected > 0)
    selected--;
  [tableView selectRow: selected byExtendingSelection: NO];
}

- (void) editRule: (id) sender
{
  int selected = [tableView selectedRow];
  if (selected < 0)
    {
      [self addRule: sender];
      return;
    }
  [[rules objectAtIndex: selected] setObject: [categoryButton stringValue]
                                      forKey: @"category"];
  [[rules objectAtIndex: selected] setObject: [compareButton stringValue]
                                      forKey: @"compare"];
  [[rules objectAtIndex: selected] setObject: [expressionField stringValue]
                                      forKey: @"expression"];
  [tableView reloadData];
}

- (id) initWithFrame: (NSRect) frame
{
  NSTableColumn *column;
  NSButton *addButton, *deleteButton;

  self = [super initWithFrame: frame];

  column = [[NSTableColumn alloc] initWithIdentifier: @"rule"];
  [column setEditable: NO];
  [column setMinWidth: 40];
  [[column headerCell] setStringValue: _(@"Rule")];

  tableView = [[ExtendedTableView alloc] initWithFrame: NSMakeRect(0, 30, 375, 120)];
  [tableView setDrawsGrid: NO];
  [tableView addTableColumn: column];
  [tableView setAllowsColumnSelection: NO];
  [tableView setTarget: self];
  [tableView setDataSource: self];
  [tableView setDelegate: self];
  [tableView setAutoresizesAllColumnsToFit: YES];
  [tableView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];
  [tableView setButtonColumnIdentifier: @"rule"];
  RELEASE(column);

  scrollView = [[NSScrollView alloc] initWithFrame: NSMakeRect(0, 30, 375, 120)];
  [scrollView setDocumentView: tableView];
  [scrollView setHasHorizontalScroller:YES];
  [scrollView setHasVerticalScroller:YES];
  [scrollView setBorderType: NSBezelBorder];

  [tableView setFrame: [scrollView documentVisibleRect]];
  
  [self addSubview: scrollView];          
  RELEASE(tableView);
  RELEASE(scrollView);

  categoryButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(5, 0, 80, 22)];
  [categoryButton setAutoenablesItems: NO];
  [categoryButton setTarget:self];
  [categoryButton setAction:@selector(categoryButtonAction:)];
  [categoryButton addItemWithTitle: _(@"Song")];
  [categoryButton addItemWithTitle: _(@"Artist")];
  [categoryButton addItemWithTitle: _(@"Album")];
  [self addSubview: categoryButton];
  RELEASE(categoryButton);

  compareButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(90, 0, 80, 22)];
  [compareButton setAutoenablesItems: NO];
  [compareButton setTarget:self];
  [compareButton setAction:@selector(compareButtonAction:)];
  [compareButton addItemWithTitle: _(@"contains")];
  [compareButton addItemWithTitle: _(@"begins with")];
  [compareButton addItemWithTitle: _(@"ends with")];
  [self addSubview: compareButton];
  RELEASE(compareButton);

  expressionField = [[NSTextField alloc] initWithFrame: NSMakeRect(175, 0, 140, 22)];
  [expressionField setDelegate: self];
  [expressionField setTarget: self];
  [expressionField setAction: @selector(editRule:)];
  [self addSubview: expressionField];
  RELEASE(expressionField);

  addButton = [[NSButton alloc] initWithFrame: NSMakeRect(325, 0, 22, 22)];
  [addButton setButtonType: NSMomentaryPushButton];
  [addButton setTitle: @"+"];
  [addButton setTarget: self];
  [addButton setAction: @selector(addRule:)];
  [self addSubview: addButton];
  RELEASE(addButton);

  deleteButton = [[NSButton alloc] initWithFrame: NSMakeRect(350, 0, 22, 22)];
  [deleteButton setButtonType: NSMomentaryPushButton];
  [deleteButton setTitle: @"-"];
  [deleteButton setTarget: self];
  [deleteButton setAction: @selector(deleteRule:)];
  [self addSubview: deleteButton];
  RELEASE(deleteButton);

  rules = [NSMutableArray new];

  return self;
}

- (void) dealloc
{
  RELEASE(rules);
  [super dealloc];
}

- (NSDictionary *) property
{
  return [NSDictionary dictionaryWithObjectsAndKeys: 
                                      [nameField stringValue], @"name",
                                      rules, @"rules",
                                      nil];

}

- (void) setProperty: (NSDictionary *) property
{
  [nameField setStringValue: [property objectForKey: @"name"]];
  [rules setArray: [property objectForKey: @"rules"]];
  [tableView reloadData];
}

- (void) tableViewSelectionDidChange: (NSNotification *) not
{
  NSDictionary *dict = [rules objectAtIndex: [[not object] selectedRow]];
  [categoryButton selectItemWithTitle: [dict objectForKey: @"category"]];
  [compareButton selectItemWithTitle: [dict objectForKey: @"compare"]];
  [expressionField setStringValue: [dict objectForKey: @"expression"]];
}

- (int) numberOfRowsInTableView: (NSTableView *) aView
{
  return [rules count];
}

- (id) tableView: (NSTableView *) aView
           objectValueForTableColumn: (NSTableColumn *) aColumn
           row: (int) row;
{
  return [NSString stringWithFormat: @"%@ %@ %@",
                [[rules objectAtIndex: row] objectForKey: @"category"],
                [[rules objectAtIndex: row] objectForKey: @"compare"],
                [[rules objectAtIndex: row] objectForKey: @"expression"]];

}

- (int) tableView: (NSTableView *) view
           stateForTableColumn: (NSTableColumn *) column
           row: (int) row
{
  if([[[rules objectAtIndex: row] objectForKey: @"state"] isEqualToString: @"YES"])
    return YES;
  else
    return NO;
}

- (void) tableView: (NSTableView *) view
          setState: (int) state
    forTableColumn: (NSTableColumn *) column
               row: (int) row
{
  if (state > 0)
    [[rules objectAtIndex: row] setObject: @"YES" forKey: @"state"];
  else
    [[rules objectAtIndex: row] setObject: @"NO" forKey: @"state"];
  [view reloadData];
}

@end

@implementation AudioSmart

+ (NSString *) classType
{
  return @"Smart Playlist";
}

+ (AudioListView *) settingView
{
  return AUTORELEASE([[AudioSmartView alloc] initWithFrame: NSMakeRect(5, 50, 240, 180)]);
}

- (BOOL) canDragSoundList
{
  return YES;
}

- (BOOL) canDropSoundList
{
  return NO;
}

- (BOOL) deleteObject: (id) object
{
  return NO;
}

- (BOOL) addObject: (id) object
{
  return NO;
}

- (BOOL) addObjects: (NSArray *) array
{
  return NO;
}

- (BOOL) insertObject: (id) object atIndex: (int) index
{
  return NO;
}

- (void) setProperty: (NSDictionary *) dict
{
  if ([dict objectForKey: @"name"])
    [self setName: [dict objectForKey: @"name"]];

  [rules setArray: [dict objectForKey: @"rules"]];

  /* Reload song list after path changed */
//  [self loadData: [self dataForSave]];
}

- (NSDictionary *) property
{
  return [NSDictionary dictionaryWithObjectsAndKeys: rules, @"rules",
                                                     nil];

}

- (id)initWithOwner: (id)owner
{
  self = [super initWithOwner: owner];
  icon = nil;
  rules = [NSMutableArray new];
//  icon = [NSImage imageNamed: @"stock_home_16.png"];
  [self setName: @"Smart Playlist"];

//  mainLibrary = [[PlaylistManager sharedManager] mainLibrary];
  mainLibrary = nil;
  return self;
}

- (BOOL) loadData: (NSArray *) aData
{
  return NO;
}

- (NSArray *) dataForSave
{
  return nil;
}

- (void)dealloc
{
  RELEASE(rules);
  [super dealloc];
}

/* Speed is much more important then memory in this method */
- (NSArray *) soundList
{
  mainLibrary = [[PlaylistManager sharedManager] mainLibrary];

  if ((rules) && ([rules count] > 0))
    {
      int i, j;
      NSArray *array = [mainLibrary soundList];
      unsigned listCount = [array count];
      unsigned ruleCount = [rules count];
      NSDictionary *rule;
      SongInfo *info;
      NSString *category, *compare, *expression;
      NSRange range;
      BOOL pass;

      [listArray removeAllObjects];
 
      for (i = 0; i < listCount; i++)
        {
          info = [array objectAtIndex: i];
          pass = YES;
          for(j = 0; j < ruleCount; j++) 
            {
              rule = [rules objectAtIndex: j];
              category = [rule objectForKey: @"category"];
              compare = [rule objectForKey: @"compare"];
              expression = [rule objectForKey: @"expression"];

              if ([category isEqualToString: _(@"Song")])
                category = [info song];
              else if ([category isEqualToString: _(@"Artist")]) 
                category = [info artist];
              else if ([category isEqualToString: _(@"Album")])
                category = [info album];

              range = [category rangeOfString: expression
                                      options: NSCaseInsensitiveSearch];

              if(range.location == NSNotFound)
                {
                  pass = NO;
                }
              else if (([compare isEqualToString: _(@"contains")]) &&
                       (range.location == NSNotFound))
                {
                  pass = NO;
                }
              else if (([compare isEqualToString: _(@"begins with")]) && 
                       (range.location != 0))
                {
                  pass = NO;
                }
              else if (([compare isEqualToString: _(@"ends with")]) && 
                       ((range.location + range.length) != [category length]))
                {
                  pass = NO;
                }
            }
          if (pass)
            {
              [listArray addObject: [info copy]];
            }
        }

      return listArray;
    }
  else
    {
      NSLog(@"AudioSmart: no rules");
      return [mainLibrary soundList];
    }
}

@end
