/*
**  IconTableView.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "IconTableView.h"
#include <AppKit/AppKit.h>

@implementation IconTableView

- (void)drawRow: (int)rowIndex clipRect: (NSRect)aRect
{
  int startingColumn;
  int endingColumn;
  NSTableColumn *tb;
  NSRect drawingRect;
  id cell;
  NSImage *icon;
  int i;
  float x_pos;

  if (_dataSource == nil)
    {
      return;
    }

  /* Using columnAtPoint: here would make it called twice per row per drawn
     rect - so we avoid it and do it natively */

  /* Determine starting column as fast as possible */
  x_pos = NSMinX (aRect);
  i = 0;
  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
    {
      i++;
    }
  startingColumn = (i - 1);

  if (startingColumn == -1)
    startingColumn = 0;

  /* Determine ending column as fast as possible */
  x_pos = NSMaxX (aRect);
  // Nota Bene: we do *not* reset i
  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
    {
      i++;
    }
  endingColumn = (i - 1);

  if (endingColumn == -1)
    endingColumn = _numberOfColumns - 1;

  /* Draw the row between startingColumn and endingColumn */
  for (i = startingColumn; i <= endingColumn; i++)
    {
      if (i != _editedColumn || rowIndex != _editedRow)
        {
          tb = [_tableColumns objectAtIndex: i];
          cell = [tb dataCellForRow: rowIndex];
          if (_del_responds)
            {
              [_delegate tableView: self   willDisplayCell: cell
                         forTableColumn: tb   row: rowIndex];
            }
     
          [cell setObjectValue: [_dataSource tableView: self
                                             objectValueForTableColumn: tb
                                             row: rowIndex]];

          drawingRect = [self frameOfCellAtColumn: i
                              row: rowIndex];
          [cell drawWithFrame: drawingRect inView: self];

          icon = [_dataSource tableView: self
                              iconForTableColumn: tb
                              row: rowIndex];
          if (icon) {
            drawingRect = NSMakeRect(drawingRect.origin.x-18,
                                     drawingRect.origin.y,
                                     16, 16);

            if ([self isFlipped])
              drawingRect.origin.y += 16;

            if (rowIndex == _selectedRow)
              {
                [icon setBackgroundColor: [NSColor highlightColor]];
              }
            else
              [icon setBackgroundColor: _backgroundColor];

            [icon compositeToPoint: drawingRect.origin
                         operation: NSCompositeCopy];
          }
        }
    }
}

- (NSRect) frameOfCellAtColumn: (int)columnIndex 
                           row: (int)rowIndex
{
  NSTableColumn *tb;
  NSString *identifier;

  if ((columnIndex < 0) 
      || (rowIndex < 0)
      || (columnIndex > (_numberOfColumns - 1))
      || (rowIndex > (_numberOfRows - 1)))
    return NSZeroRect;

  tb = [_tableColumns objectAtIndex: columnIndex];
  identifier = [tb identifier];

  /* FIXME: Only work for "source" */
  if ([identifier isEqual: @"source"])
    {
      NSRect result;
      result = [super frameOfCellAtColumn: columnIndex
                      row:rowIndex];
      result.origin.x += 18;
      result.size.width -= 18;
      return result;
    }
  else
    {
      return [super frameOfCellAtColumn: columnIndex
                    row: rowIndex];
    }
}

@end
