/*
**  LED.m
**
**  Copyright (c) 2002
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ShengGuang/SongInfo.h>
#include <ShengGuang/SGSound.h>
#include "LED.h"
#include "ActionController.h"
#include <AppKit/AppKit.h>

@implementation LED

/* Drop from playlist manager */
- (NSDragOperation) draggingEntered: (id <NSDraggingInfo>) sender
{
  return NSDragOperationAll;
}

- (BOOL) prepareForDragOperation: (id <NSDraggingInfo>) sender
{
  return YES;
}

- (BOOL) performDragOperation: (id <NSDraggingInfo>) sender
{
  /* The selection in playlist window must be correct,
     so don't need to worry about that, just play */

  if (nil == [sender draggingSource]) // From other application
    {
      return NO;
    }
  else
    {
//      if ([actionController isEncoding] == NO)
        {
          if ( ([actionController currentState] == ActionControllerPlaying) ||
               ([actionController currentState] == ActionControllerPaused) )
            [actionController stop: self];
          else
            [actionController play: self];
        }
        return YES;
    }
}

/* Handle stop and dragging progressive bar */
- (void) mouseDown: (NSEvent *) event
{
  NSEvent *nextEvent;
  NSPoint location = [self convertPoint: [event locationInWindow]
                       fromView: nil];
  
  while (YES)
    {
      NSPoint newLocation;

      nextEvent = [_window nextEventMatchingMask: NSLeftMouseUpMask | NSLeftMouseDraggedMask];
      newLocation = [self convertPoint: [nextEvent locationInWindow]
                                          fromView: nil];

      if (NSMouseInRect(location, barFrame, NO))
        {
          int currentTime = 0;

          bar = 100.0*(newLocation.x-5)/(barFrame.size.width);
          if (bar < 0) bar = 0;
          else if (bar > 100) bar = 100;

          currentTime = (bar*totalTime)/100.0;
          [timeLine setString: [NSString stringWithFormat: @"%02d:%02d",
                                       (currentTime/60), 
                                       (currentTime%60)]];
        }
      
      if ( [nextEvent type] == NSLeftMouseUp )
        {
          [self mouseUp: event];
          break;
        }
      else if ( [nextEvent type] == NSLeftMouseDragged )
        {
          if ((NSMouseInRect(location, barFrame, NO)) && (canDrag))
            {
              isDragging = YES;
              [self display];
            }
          else
           {
             [super mouseDown: nextEvent];
           }
        }
    }
}

- (void) mouseUp: (NSEvent *) event
{
  NSRect stopRect;
  NSPoint location = [self convertPoint: [event locationInWindow]
                       fromView: nil];

  stopRect = [self frame];
  stopRect = NSMakeRect(stopRect.size.width-17, 17, 17, 17);

  if ((NSMouseInRect(location, stopRect, NO)) && (hasStop == YES))
    {
      [actionController stop: self];
    }
  else if (NSMouseInRect(location, barFrame, NO))
    {
      [actionController seek: [NSNumber numberWithInt: (bar*totalTime)/100.0]
                          by: self];
      isDragging = NO;
    }
  [super mouseUp: event];
}

- (void) setFrame: (NSRect) frame
{
  [super setFrame: frame];

  if (frame.size.height < 52)
    {
      firstSize = secondSize = (frame.size.height-24)/2.0;
    }
  else 
    {
      firstSize = (frame.size.height-38);
      secondSize = 12;
    }

  [firstAttr setObject: [NSColor whiteColor]
                forKey: @"NSForegroundColorAttributeName"];
  [firstAttr setObject: [NSFont systemFontOfSize: firstSize]
                forKey: @"NSFontAttributeName"];

  [secondAttr setObject: [NSColor whiteColor]
                 forKey: @"NSForegroundColorAttributeName"];
  [secondAttr setObject: [NSFont systemFontOfSize: secondSize]
                 forKey: @"NSFontAttributeName"];

  barFrame = NSMakeRect(5, 6, frame.size.width - 47, 11);
}

- (id) initWithFrame: (NSRect) frame;
{
  self = [super initWithFrame: frame];

  firstLine = [NSMutableString new];
  secondLine = [NSMutableString new];
  timeLine = [NSMutableString new];
  firstAttr = [NSMutableDictionary new];
  secondAttr = [NSMutableDictionary new];
  timeAttr = [NSMutableDictionary new];

  [timeAttr setObject: [NSColor whiteColor]
                 forKey: @"NSForegroundColorAttributeName"];
  [timeAttr setObject: [NSFont systemFontOfSize: 12]
                 forKey: @"NSFontAttributeName"];

  percent = 0;
  totalTime = 0;
  isDragging = NO;
  hasStop = NO;
  canDrag = NO;
  currentState = ActionControllerStopped;

  gnuspace = [NSImage imageNamed: @"GNUSpace.tiff"];
  RETAIN(gnuspace);

  iconSize = [gnuspace size];
  actionController = [ActionController sharedController];

  [self registerForDraggedTypes: [NSArray arrayWithObjects: NSGeneralPboardType, nil]];

  return self;
}

- (void) dealloc
{
  [self unregisterDraggedTypes];
  RELEASE(gnuspace);
  RELEASE(firstLine);
  RELEASE(secondLine);
  RELEASE(timeLine);
  RELEASE(firstAttr);
  RELEASE(secondAttr);
  RELEASE(timeAttr);
  [super dealloc];
}

- (void) setState: (ActionControllerState) state
{
  ActionControllerState oldState = currentState;
  currentState = state;
  if (currentState == ActionControllerStopped)
    {
      [self setNeedsDisplay: YES];
      return;
    }
  if (currentState == ActionControllerPlaying)
    {
      int currentTime;
      SongInfo *song = [actionController playingSongInfo];
      [firstLine setString: [song song]];
      [secondLine setString: [NSString stringWithFormat: @"%@, %@", 
                                                [song artist],
                                                [song album]]];
      totalTime = [[song duration] intValue];
      currentTime = [[(SGSound *)[song sound] currentTime] intValue];
      [timeLine setString: [NSString stringWithFormat: @"%02d:%02d", 
                                         (currentTime/60), 
                                         (currentTime%60)]];
      if ((totalTime < 1) || (currentTime > totalTime))
        percent = 100;
      else
        percent = (currentTime * 100/totalTime);
      hasStop = NO;
      canDrag = YES;
      [self display];
    }
}

- (void) drawRect: (NSRect) rect
{
  NSPoint point;
  NSBezierPath *path;

  if([_window isVisible] == NO)
    return;

  path = [NSBezierPath bezierPath];

  [super drawRect: rect];

  [[NSColor darkGrayColor] set];
  NSRectFill(rect);
  
  [[NSColor whiteColor] set];
  [path moveToPoint: NSMakePoint(1,1)];
  [path lineToPoint: NSMakePoint(rect.size.width - 1, 1)];
  [path lineToPoint: NSMakePoint(rect.size.width - 1, rect.size.height)];
  [path stroke];

  /* Draw the title/artist/album, or the GNUstep symbol */  
  if (currentState == ActionControllerStopped) {
    point = NSMakePoint((rect.size.width-iconSize.width)/2.0, 
                         (rect.size.height-iconSize.height)/2.0);
    [gnuspace compositeToPoint: point operation: NSCompositeSourceOver];
  } else {
    point = NSMakePoint(barFrame.origin.x,
             (barFrame.origin.y+barFrame.size.height+4+secondSize+2+firstSize));
    [firstLine drawAtPoint: point withAttributes: firstAttr];
    point = NSMakePoint(barFrame.origin.x, 
                        (barFrame.origin.y+barFrame.size.height+4+secondSize));
    [secondLine drawAtPoint: point withAttributes: secondAttr];

    /* Draw the stop button or time*/
    if (hasStop == YES)
      {
        [path setLineWidth: 2];
        [path moveToPoint: NSMakePoint(rect.size.width - 15, 18 )];
        [path lineToPoint: NSMakePoint(rect.size.width - 7, 26)];
        [path stroke];
        [path moveToPoint: NSMakePoint(rect.size.width - 7, 18)];
        [path lineToPoint: NSMakePoint(rect.size.width - 15, 26)];
        [path stroke];
      }
//    else
      {
        point = NSMakePoint(barFrame.size.width+barFrame.origin.x+5,
                            barFrame.origin.y+12);
        [timeLine drawAtPoint: point withAttributes: timeAttr];
      }

    /* Draw the progress bar */
    NSFrameRect(barFrame);

    [[NSColor grayColor] set];
    NSRectFill(NSMakeRect(barFrame.origin.x+1,
                          barFrame.origin.y+1,
                         (barFrame.size.width-2)*percent/100,
                          barFrame.size.height-2));

    /* draw the slider */
    if(canDrag)
      {
        if(isDragging == NO)
          bar = percent;


        [[NSColor whiteColor] set];

        [path setLineWidth: 1];
        [path moveToPoint: NSMakePoint(barFrame.origin.x + 
                                       (barFrame.size.width-1)*bar/100, 16)];
        [path relativeLineToPoint: NSMakePoint(-2, -4.5)];
        [path relativeLineToPoint: NSMakePoint(2, -4.5)];
        [path relativeLineToPoint: NSMakePoint(2, 4.5)];
        [path closePath];
        [path stroke];
      }
  }
}

@end
