/*
**  MusicBox.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _Musicbox_MusicBox_H_MusicBox
#define _Musicbox_MusicBox_H_MusicBox

#include <Foundation/NSObject.h>
#include <AppKit/AppKitDefines.h>

@class TagEditorPanelController;
@class PlayerWindowController;
@class PlaylistWindowController;
@class MusicBoxWindowController;
@class PlaylistManager;
@class ActionController;
@class PlayerView;
@class LED;
@class NSNotification;
@class NSString;

@interface MusicBox: NSObject
{
  PlayerView *playerView;
  PlayerWindowController *playerController;
  PlaylistWindowController *playlistController;
  PlaylistManager *playlistManager;
  ActionController *actionController;
  TagEditorPanelController *tagEditorController;
  MusicBoxWindowController *mainWindowController;
  LED *led;
}

/* Open the window if not exist */
- (void) playlistWindow: (id) sender;
 - (void) playerWindow: (id) sender;

// Delegate method for NSApp 
- (void) applicationWillFinishLaunching: (NSNotification *) not;
- (void) applicationDidFinishLaunching: (NSNotification *) not;
- (BOOL) applicationShouldTerminate: (id) sender;
- (void) applicationWillTerminate: (NSNotification *)not;

// Called by NSMenu
- (void) showPrefWindow: (id)sender;
- (void) editTag: (id) sender;

// Notification

/* This method is used when the browser and tableView
 * need to refresh themselves, ex. songs deleted.
 * object: nil
 */
APPKIT_EXPORT NSString *MBMusicBoxWindowNeedReloadNotification;

/* Reload the table in musicbox window
 * object: nil
 */
APPKIT_EXPORT NSString *MBShouldReloadTableViewData;

/* Reload the table in playlist window
 * object: nil
 */
APPKIT_EXPORT NSString *MBShoudReloadPlaylistNotification;

/* For connection between tableview and tag editor */
APPKIT_EXPORT NSString *MBSelectedSongInfoChanged;

/* For connection between actionController and the playback control 
 * Several keyword can be got from [notification object]:
 * @"Encoding", @"EncodingStopped", @"Stopped", @"Playing", @"Paused"
 */
APPKIT_EXPORT NSString *MBControlStatusChanged;

/* Notify the selected changed from ActionController 
 * to MusicBoxWindowController
 */
APPKIT_EXPORT NSString *MBSelectedSongInfoShouldChange;

@end

#endif /* _Musicbox_MusicBox_H_MusicBox */
