/*
**  MusicBox.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MusicBox.h"
#include <MusicBox/ModuleLoader.h>
#include <MusicBox/AudioList.h>
#include "PlaylistManager.h"
#include "PlaylistWindowController.h"
#include "ActionController.h"
#include "PreferenceWindowController.h"
#include "MusicBoxWindowController.h"
#include "PlayerWindowController.h"
#include "PlayerView.h"
#include "LED.h"
#include "TagEditorPanelController.h"

@implementation MusicBox

- (void) editTag: (id) sender
{
  [tagEditorController showWindow: sender];
}

/*
- (BOOL)validateMenuItem: (NSMenuItem*) menuItem
{
  if ([[musicboxController window] isVisible] == NO)
    {
      if (([[menuItem title] isEqualToString: _(@"New Play List")]) ||
          ([[menuItem title] isEqualToString: _(@"Edit Play List")]) ||
          ([[menuItem title] isEqualToString: _(@"Delete Play List")]) ||
          ([[menuItem title] isEqualToString: _(@"Import Song")]) ||
          ([[menuItem title] isEqualToString: _(@"Delete Song")]))
        return NO;
    }
  return YES;
}
*/

- (void) playlistWindow: (id) sender
{
  [[playlistController window] makeKeyAndOrderFront: sender];
}

- (void) playerWindow: (id) sender
{
  [[playerController window] makeKeyAndOrderFront: sender];
}

- (void) showPrefWindow: (id)sender
{
  NSAutoreleasePool *pool;
  PreferenceWindowController *prefController;
  NSWindow *window;

  pool = [[NSAutoreleasePool alloc] init];
  prefController = [[PreferenceWindowController alloc] initWithWindowNibName: @"PreferenceWindow"];
  window = [prefController window];
  
  if (window)
    {
      [NSApp runModalForWindow: window];
    }

  RELEASE(prefController);
  RELEASE(pool);
}

- (void) applicationDidFinishLaunching: (NSNotification *) not
{
  Class moduleClass;

//  [NSApp setServicesProvider: self];

  /* Set up the basic control unit */
  playlistManager = [PlaylistManager sharedManager];
  actionController = [ActionController sharedController];

/* Init Tag Editor. Must before main window in order to receive 
   the first notification of selected song info */

  tagEditorController = [[TagEditorPanelController alloc] initWithWindowNibName: @"TagEditorPanel"];

  /* Load LED module */

  led = [[LED alloc] initWithFrame: NSMakeRect(10, 10, 200, 50)];

  /* Set up playlist window */
  playlistController = [[PlaylistWindowController alloc] initWithWindowNibName: @"PlaylistWindow"];
  [[playlistController window] orderFront: self];

  /* Set up player */
  playerController = [[PlayerWindowController alloc] initWithWindowNibName: @"PlayerWindow"];
  playerView = [[PlayerView alloc] initWithFrame: [[[playerController window] contentView] bounds]];
  [[playerController window] setContentView: playerView];
  [[playerController window] orderFront: self];
  RELEASE(playerView);

  /* Set up main window */
  mainWindowController = [[MusicBoxWindowController alloc] initWithWindowNibName: @"MusicBoxWindow"];
  NSLog(@"playlistManager %@", [playlistManager selectedPlaylist]);
  [mainWindowController setAudioList: [playlistManager selectedPlaylist]];
  [[mainWindowController window] setTitle: [[playlistManager selectedPlaylist] name]];
  [[mainWindowController window] makeKeyAndOrderFront: self];

  /* Cross relate to each other */
  [actionController setLED: led];
  [playerView setLED: led];
  RELEASE(led);
}

- (void) applicationWillFinishLaunching: (NSNotification *) not
{
  NSMenu *menu;
  NSMenu *infoMenu;
  NSMenu *editMenu;
  NSMenu *controlMenu;
  NSMenu *selectMenu;
  NSMenu *servicesMenu;
  NSMenu *windowsMenu;
  
  menu = [[NSMenu alloc] initWithTitle: _(@"MusicBox")];

  [menu addItemWithTitle: _(@"Info")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Edit")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Control")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Select")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Windows")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Services")
                  action: NULL
           keyEquivalent: @""];
  [menu addItemWithTitle: _(@"Hide")
                  action: @selector(hide:)
           keyEquivalent: @"h"];
  [menu addItemWithTitle: _(@"Quit")
                  action: @selector(terminate:)
           keyEquivalent: @"q"];

  // Create the info submenu
  infoMenu = [NSMenu new];
   
  [infoMenu addItemWithTitle: _(@"Info Panel...")
                      action: @selector(orderFrontStandardInfoPanel:)
               keyEquivalent: @"i"];
  [infoMenu addItemWithTitle: _(@"Preferences...")
                      action: @selector(showPrefWindow:)
               keyEquivalent: @"p"];
  [infoMenu addItemWithTitle: _(@"Help")
                      action: @selector(orderFrontHelpPanel:)
               keyEquivalent: @"?"];

  [menu setSubmenu: infoMenu
           forItem: [menu itemWithTitle: _(@"Info")]];

  RELEASE(infoMenu);

  // Create the edit submenu
  editMenu = [NSMenu new];

  [editMenu addItemWithTitle: _(@"New Play List")
                      action: @selector(addPlaylist:)
               keyEquivalent: @""]; 
 
  [editMenu addItemWithTitle: _(@"Edit Play List")
                      action: @selector(editPlaylist:)
               keyEquivalent: @""];
  
  [editMenu addItemWithTitle: _(@"Delete Play List")
                      action: @selector(deletePlaylist:)
               keyEquivalent: @""];

  [editMenu addItemWithTitle: _(@"Import Song")
                      action: @selector(importSong:)
               keyEquivalent: @""];

  [editMenu addItemWithTitle: _(@"Delete Song")
                      action: @selector(deleteSong:)
               keyEquivalent: @""];

  [editMenu addItemWithTitle: _(@"Edit Tag")
                      action: @selector(editTag:)
               keyEquivalent: @"t"];

  [menu setSubmenu: editMenu
           forItem: [menu itemWithTitle: _(@"Edit")]];

  RELEASE(editMenu);

  // Create the control submenu
  controlMenu = [NSMenu new];
  
  [controlMenu addItemWithTitle: _(@"Playlist Manager")
                         action: @selector(playlistWindow:)
                  keyEquivalent: @""];
  
  [controlMenu addItemWithTitle: _(@"Player Window")
                         action: @selector(playerWindow:)
                  keyEquivalent: @""];

  [menu setSubmenu: controlMenu
           forItem: [menu itemWithTitle: _(@"Control")]];

  RELEASE(controlMenu);

  // Create the select menu
  selectMenu = [NSMenu new];

  [selectMenu addItemWithTitle: _(@"Select All")
                        action: @selector(selectAllSong:)
                 keyEquivalent: @""];

  [selectMenu addItemWithTitle: _(@"Select None")
                        action: @selector(selectNoneSong:)
                 keyEquivalent: @""];

  [selectMenu addItemWithTitle: _(@"Select Inverse")
                        action: @selector(selectInverseSong:)
                 keyEquivalent: @""];

  [menu setSubmenu: selectMenu
           forItem: [menu itemWithTitle: _(@"Select")]];

  RELEASE(selectMenu);

  // Create the windows submenu
  windowsMenu = [NSMenu new];

  [windowsMenu addItemWithTitle: _(@"Arrange")
                     action: @selector(arrangeInFront:)
              keyEquivalent: @""];
  [windowsMenu addItemWithTitle: _(@"Miniaturize")
                     action: @selector(performMiniaturize:)
              keyEquivalent: @"m"];
  [windowsMenu addItemWithTitle: _(@"Close")
                     action: @selector(performClose:)
              keyEquivalent: @"w"];

  [menu setSubmenu: windowsMenu
           forItem: [menu itemWithTitle: _(@"Windows")]];
  [NSApp setWindowsMenu: windowsMenu];
  RELEASE(windowsMenu);

  // Create the service submenu
  servicesMenu = [NSMenu new];
  [menu setSubmenu: servicesMenu
           forItem: [menu itemWithTitle: _(@"Services")]];
  [NSApp setServicesMenu: servicesMenu];
  RELEASE(servicesMenu);

  [NSApp setMainMenu: menu];

  RELEASE(menu);
}

- (BOOL) applicationShouldTerminate: (id)sender
{
  // Stop play sound
  [actionController stop: self];

  return YES;
}

- (void) applicationWillTerminate: (NSNotification *) not
{
  [playlistManager writeToFile];
}

- (id) init
{
  self = [super init];

  playlistManager = nil;
  actionController = nil;

  return self;
}

- (void) dealloc
{
  RELEASE(playlistManager);
  RELEASE(actionController);
  RELEASE(playerController);
  RELEASE(playlistController);
  RELEASE(tagEditorController);
  RELEASE(mainWindowController);
  [super dealloc];
}

@end
