/*
**  MusicBoxWindowController.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _Musicbox_MusicBox_H_MusicBoxWindowController
#define _Musicbox_MusicBox_H_MusicBoxWindowController

#include <AppKit/AppKit.h>

@class ExtendedTableView;
@class PlaylistManager;
@class ActionController;
@class AudioList;
@class SongInfo;

@interface MusicBoxWindowController : NSWindowController
{
  /* Interface */
  ExtendedTableView *tableView;
  NSScrollView *scrollView;
  NSBrowser *browser;
  NSSplitView *splitView;
  NSRect frameOfBrowser, frameOfScrollView;
  NSButton *browserButton;
  NSButton *repeatButton, *shuffleButton;
  NSTextField *statusLabel, *searchField;
  NSButton *additionalButton;

  /* Controller */
  PlaylistManager *playlistManager;
  ActionController *actionController;

  /* Modal */
  NSMutableString *selectedPath;
  NSMutableArray *listForDisplay;
  AudioList *audioList;
  int selectedSongForDisplay;
}

- (SongInfo *) selectedSongInfo;
- (void) setAudioList: (AudioList *) list;
- (AudioList *) audioList;
- (NSArray *) listForDisplay;
- (int) selectedSongForDisplay;

- (void) updateStateBar;
- (void) browserView: (id) sender;

/* menu action */
- (void) importSong: (id) sender;
- (void) deleteSong: (id) sender;

- (void) selectAllSong: (id) sender;
- (void) selectNoneSong: (id) sender;
- (void) selectInverseSong: (id) sender;

- (void) reloadBrowserAndTableView: (NSNotification *) not;

/* active delegate of browser */
- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column
                                        inMatrix:(NSMatrix *)matrix;
- (BOOL) browser: (NSBrowser *)sender selectRow: (int)row 
                                       inColumn: (int)column;
- (NSString *) browser: (NSBrowser *)sender titleOfColumn: (int)column;

- (void) browserAction: (id) sender;

/* delegate of ExtendedTableView */
- (void) tableViewSelectionDidChange: (NSNotification *)aNotification;
/*   Drop */
- (BOOL) tableView: (NSTableView*)tableView
        acceptDrop: (id <NSDraggingInfo>)info
               row: (int)row
     dropOperation: (NSTableViewDropOperation)operation;
- (NSDragOperation) tableView: (NSTableView*)tableView
                 validateDrop: (id <NSDraggingInfo>)info
                  proposedRow: (int)row
        proposedDropOperation: (NSTableViewDropOperation)operation;
/*   Drag */
- (BOOL) tableView: (NSTableView *) aView
         writeRows: (NSArray *) rows
      toPasteboard: (NSPasteboard *) pboard;
/*   Data Source */
- (int) numberOfRowsInTableView: (NSTableView *)aView;
- (id) tableView: (NSTableView *) aView
                  objectValueForTableColumn: (NSTableColumn *) aColumn
                  row: (int) row;
- (int) tableView: (NSTableView *) aView
                   stateForTableColumn: (NSTableColumn *)aColumn
                   row: (int)rowIndex;
- (void) tableView: (NSTableView *) aView
                    setState: (int) aState
                    forTableColumn: (NSTableColumn *)aColumn
                    row: (int) row;
- (void) reloadTableViewData: (NSNotification *) not;
- (void) selectRowInTableView: (NSNotification *) not;
/*    Action */
- (void) doubleClickOnTable: (id) sender;

/* delegate of splitView */
- (BOOL) splitView: (NSSplitView *)sender
         canCollapseSubview: (NSView *)subview;
- (float) splitView: (NSSplitView *)sender
          constrainMaxCoordinate: (float)proposedMax
          ofSubviewAt: (int)offset;
- (float) splitView: (NSSplitView *)sender
          constrainMinCoordinate: (float)proposedMin
          ofSubviewAt: (int)offset;

/* delegate of searchField */
- (void) controlTextDidChange: (NSNotification *) not;
- (void) setSearch: (id) sender;

/* delegate of window */
- (void) windowDidBecomeMain: (NSNotification *) not;

@end

#endif /* _Musicbox_MusicBox_H_MusicBoxWindowController */
