/*
**  MusicBoxWindowController.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MusicBoxWindowController.h"
#include <ShengGuang/SongInfo.h>
#include <MusicBox/ModuleLoader.h>
#include <MusicBox/AudioList.h>
#include <MusicBox/ExtendedTableView.h>
#include "PlaylistManager.h"
#include "ActionController.h"

@implementation MusicBoxWindowController

- (void) selectAllSong: (id) sender
{
  int i, count = [listForDisplay count];

  for (i = 0; i < count; i++)
    {
      [[listForDisplay objectAtIndex: i] setChecked: YES];
      [tableView reloadData];
    }
}

- (void) selectNoneSong: (id) sender
{
  int i, count = [listForDisplay count];
  
  for (i = 0; i < count; i++)
    {
      [[listForDisplay objectAtIndex: i] setChecked: NO];
      [tableView reloadData];
    }
}

- (void) selectInverseSong: (id) sender
{
  id object;
  int i, count = [listForDisplay count];

  for (i = 0; i < count; i++)
    {
      object = [listForDisplay objectAtIndex: i];

      [object setChecked: (![object checked])];

      [tableView reloadData];
    }
}

- (void) reloadBrowserAndTableView: (NSNotification *) not
{
  [[self window] setTitle: [audioList name]];
  [browser loadColumnZero];
  if ([browser selectedRowInColumn: 0] < 0)
    [browser selectRow: 0 inColumn: 0];
}

- (void) windowDidBecomeMain: (NSNotification *) not
{
  [playlistManager setMainWindowController: self];
}

- (SongInfo *) selectedSongInfo
{
  return [listForDisplay objectAtIndex: selectedSongForDisplay];
}

// browser button pressed
- (void) browserView: (id) sender
{
  /* ActionController will hide browser before play/next/prev */
  if (sender == actionController)
    [browserButton setState: NO];

  if([browserButton state]) // Show browser
    {
      RETAIN(scrollView);
      [scrollView removeFromSuperview];
      [browser setFrame: frameOfBrowser];
      [scrollView setFrame: frameOfScrollView];
      [splitView addSubview: browser];
      [splitView addSubview: scrollView];
      RELEASE(scrollView);
      RELEASE(browser);
      NSLog(@"finished");
    }
  else // Hide browser
    {
      [browser selectRow: 0 inColumn: 0];
      frameOfBrowser = [browser bounds];
      frameOfScrollView = [scrollView bounds];
      RETAIN(browser);
      [browser removeFromSuperview];
      [searchField setStringValue: @""];
    }
  [splitView adjustSubviews];
  [splitView setNeedsDisplay: YES];
}

// delegate of splitView
- (BOOL) splitView: (NSSplitView *)sender
         canCollapseSubview: (NSView *)subview
{
  /* FIXME: No implement yet in GNUstep */
  return YES;
}

- (float) splitView: (NSSplitView *)sender
          constrainMaxCoordinate: (float)proposedMax
          ofSubviewAt: (int)offset
{
  return [sender frame].size.height*2/3;
}

- (float) splitView: (NSSplitView *)sender
          constrainMinCoordinate: (float)proposedMin
          ofSubviewAt: (int)offset
{
  return [sender frame].size.height/5;
}

// Active delegate of browser
- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column
                                        inMatrix:(NSMatrix *)matrix
{
  int i, count;
  NSArray *array;
  NSString *realPath;

  realPath = [sender pathToColumn: column];
  array = [audioList filterListWithPath: realPath];
  count = [array count];

  // Nothing 
  if (count == 0)
    {
      [selectedPath setString: @""];
      [listForDisplay setArray: [audioList soundList]];
      [tableView reloadData];
      return;
    }

  [matrix addColumn];
  for (i = 0; i < count; i++)
    {
      id cell;

      // First row is created when column is added
      if (i != 0)
        [matrix insertRow: i];
  
      cell = [matrix cellAtRow: i column: 0];
      [cell setStringValue: [array objectAtIndex: i]];
      [cell setLeaf: ![audioList containMoreKeys:
                        [realPath stringByAppendingPathComponent:
                                                    [array objectAtIndex: i]]]];

    }
}

/* Delegate is responsible for select cell and return YES */
- (BOOL) browser: (NSBrowser *)sender selectRow: (int)row inColumn: (int)column
{
  int i;

  [selectedPath setString: @""];

  /* Got the right path depending on the selection,
   * ignore the 1st column
   */

  for(i = 0; i < column; i++) {
    if ([browser selectedRowInColumn: i] > -1)
      {
        [selectedPath appendFormat: @"/%@", [[browser selectedCellInColumn: i] stringValue]];
      }
  }

  [selectedPath appendFormat: @"/%@", [[browser loadedCellAtRow: row column: column] stringValue]];

  [listForDisplay setArray: [audioList soundListWithPath: selectedPath]];

  /* Select cell */
  [[browser matrixInColumn: column] selectCellAtRow: row
                                             column: 0];
  [tableView reloadData];

  if ([listForDisplay count] > 0)
    {
      [tableView selectRow: 0 byExtendingSelection: NO];
    }

  [self updateStateBar];

  return YES;
}

- (NSString *) browser: (NSBrowser *)sender titleOfColumn: (int)column
{
 NSArray *keys = [audioList allKeys];

  /* Use the pre-defined keys */
  if (keys) 
    {
      if (column > [keys count]-1)
        return @"";
      return [keys objectAtIndex: column];
    }
  /* Use the string value of preceding column */
  return [[sender selectedCellInColumn: column-1] stringValue];
}

- (void) browserAction: (id) sender
{
  [self browser: sender
      selectRow: [sender selectedRowInColumn: [sender selectedColumn]]
       inColumn: [sender selectedColumn]];
}

- (NSArray *) listForDisplay
{
  return listForDisplay;
}

- (int) selectedSongForDisplay
{
  return selectedSongForDisplay;
}

- (AudioList *) audioList
{
  return audioList;
}

- (void) setAudioList: (AudioList *) list;
{
  NSRect frame = [[[self window] contentView] frame];

  ASSIGN(audioList, list);
  [listForDisplay setArray: [audioList soundList]];

  [tableView reloadData];

  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBMusicBoxWindowNeedReloadNotification"
                         object: nil];

  // Ensure tag panel get the right song information 
  if ([listForDisplay count] > 0)
    [[NSNotificationCenter defaultCenter]
             postNotificationName: @"MBSelectedSongInfoChanged"
             object: [self selectedSongInfo]];
  else
    [[NSNotificationCenter defaultCenter]
             postNotificationName: @"MBSelectedSongInfoChanged"
             object: nil];

  if (additionalButton)
    [additionalButton removeFromSuperview];

  additionalButton = [audioList additionalButtonOnMusicBoxWindow];
  if (additionalButton)
    {
      [additionalButton setFrame: NSMakeRect(frame.size.width-72, frame.size.height-36, 32, 32)];
      [additionalButton setAutoresizingMask: NSViewMinXMargin | NSViewMinYMargin];
      [[[self window] contentView] addSubview: additionalButton];
    }
}

- (void) importSong: (id)sender
{
/*
  int i;

  if([[[audioList class] classType] isEqualToString: @"Audio CD"] == NO)
    {
      return;
    }

  if (![[NSUserDefaults standardUserDefaults] objectForKey: @"EncoderToUse"])
    {
      NSRunAlertPanel(_(@"Encoder error"),
                      _(@"Warning: The encoder is not set up properly."),
                                   _(@"OK"),
                                   nil,
                                   nil);
      return;
    }
*/
  /* Use the facility of play to encoding */
//  [playlistManager syncPlaylist];

  /* Search for the first checked song */
/*
  for (i = 0; i < [[playlistManager listForPlay] count]; i++)
    {
      if ([[playlistManager songInfoForPlayAtIndex: i] checked] == YES)
        {
          [actionController encodeSong: i];
          break;
        }
    }
*/
}
  
- (void) deleteSong: (id)sender
{
  NSArray *array = [[tableView selectedRowEnumerator] allObjects];
  int i;

  for (i = [array count] - 1; i > -1; i--)
    {
      [audioList deleteObject: [[audioList soundList] objectAtIndex:
                                   [[array objectAtIndex: i] intValue]]];
    }

  /* Reload browser and tableView */
  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBMusicBoxWindowNeedReloadNotification"
                         object: nil];
}

// Delegate of searchField

- (void) setSearch: (id) sender
{
  /* the path is decided when selectRow: */
  NSMutableArray *array = nil;

  [listForDisplay setArray: [audioList soundListWithPath: selectedPath]];

  if (([sender stringValue] == nil) ||
      ([[[sender stringValue] stringByTrimmingSpaces] isEqualToString: @""]))
  {
    array = [NSMutableArray arrayWithArray: listForDisplay];
  }
  else
  {
    int i;
    array = [NSMutableArray new];

    for (i = 0; i < [listForDisplay count]; i++) {
      NSRange range;
      range = [[[listForDisplay objectAtIndex: i] description] rangeOfString: [sender stringValue]
                                                          options: NSCaseInsensitiveSearch];
      if (range.location != NSNotFound) {
        [array addObject: [listForDisplay objectAtIndex: i]];
      }
    }
    AUTORELEASE(array);
  }

  [listForDisplay setArray: array];
  [self updateStateBar];
  [tableView reloadData];
}

- (void) controlTextDidChange: (NSNotification *) not
{
  if ([not object] == searchField)
    [self setSearch: [not object]];
}

// ExtendedTableView delegate
- (void) tableViewSelectionDidChange: (NSNotification *) not
{
  selectedSongForDisplay = [tableView selectedRow];
  [tableView scrollRowToVisible: selectedSongForDisplay];

  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBSelectedSongInfoChanged"
           object: [self selectedSongInfo]];

/*
  if (([actionController isPlaying]) || ([actionController isEncoding]))
    {
    }
  else
    {
      if ([self selectedSongInfo])
        {
          NSDictionary *dict = [NSDictionary dictionaryWithObject: [self selectedSongInfo]
                                                           forKey: @"song"];
        }
    }
*/
}

- (void) doubleClickOnTable: (id) sender
{
  if ([tableView clickedColumn] == 0)
    {
      // Syng display list with play list 
      [playlistManager syncPlaylist];
      [actionController doubleClickAction: self];
    }
/*
  else if([[[[tableView tableColumns] objectAtIndex: [tableView clickedColumn]]
                         identifier] isEqualToString: @"duration"])
    {
      int result = NSRunAlertPanel(_(@"Update duration of song"),
                                   _(@"Warning: Decode takes long time"),
                                   _(@"Cancel"),
                                   _(@"Decode"),
                                   nil);
      if(result == NSAlertAlternateReturn)
        [[listForDisplay objectAtIndex: [tableView clickedRow]] updateDuration];
    }
*/
}

- (BOOL) tableView: (NSTableView*)view
        acceptDrop: (id <NSDraggingInfo>)info
               row: (int)row
     dropOperation: (NSTableViewDropOperation)operation
{
  NSPasteboard *pboard;
  NSData *data;

  pboard = [info draggingPasteboard];
  data = [pboard dataForType: @"NSGeneralPboardType"];

  if (nil == [info draggingSource]) // From other application
    {
      return NO;
    }
  else if (tableView == [info draggingSource])
    {
      // Use delete and insert object 
      NSArray *array, *rows;
      BOOL result;
      int i, dest = row;

      array = [NSUnarchiver unarchiveObjectWithData: data];
      rows = [[tableView selectedRowEnumerator] allObjects];

      // Delete first 
      for (i = [rows count] - 1; i > -1; i--)
        {
          int index = [[rows objectAtIndex: i] intValue];
          [audioList deleteObject: [listForDisplay objectAtIndex: index]];
 
          if (index < dest)
            dest--;
        }

      for (i = [array count] - 1; i > -1; i--)
        {
          [audioList insertObject: [array objectAtIndex: i] 
                          atIndex: dest];
        }

      [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBMusicBoxWindowNeedReloadNotification"
                         object: nil];
      return YES;
    }
  else // From playlist window
    {
      return NO;
    }
}

- (NSDragOperation) tableView: (NSTableView*)view
                 validateDrop: (id <NSDraggingInfo>)info
                  proposedRow: (int)row
        proposedDropOperation: (NSTableViewDropOperation)operation
{
  [view setDropRow: row dropOperation: NSTableViewDropAbove];

  if (row > [audioList count] - 1)
    return NSDragOperationNone;

  if (nil == [info draggingSource]) // From other application
    {
      return NSDragOperationNone;
    }
  else if (tableView == [info draggingSource])
    {
      return NSDragOperationMove;
    }
  else // Come from playlist window
    {
      return NSDragOperationNone;
    }
}

- (BOOL) tableView: (NSTableView *) view writeRows: (NSArray *)rows
         toPasteboard: (NSPasteboard *) pboard
{
  NSMutableArray *array;
  NSData *data;
  int i;

  if ([audioList canDragSoundList] == NO)
    return NO;

  array = [NSMutableArray new];
  for(i = 0; i < [rows count]; i++)
    [array addObject: [listForDisplay objectAtIndex: 
                                    [[rows objectAtIndex: i] intValue]]];

  data = [NSArchiver archivedDataWithRootObject: array];

  [pboard declareTypes: [NSArray arrayWithObject: @"NSGeneralPboardType"]
                 owner: nil];
  [pboard setData: data forType: @"NSGeneralPboardType"];

  return YES;
}

- (int) numberOfRowsInTableView: (NSTableView *) view
{
  return [listForDisplay count];
}

- (id) tableView: (NSTableView *) view
           objectValueForTableColumn: (NSTableColumn *) column
           row: (int) row;
{
  SongInfo *info = [listForDisplay objectAtIndex: row];

  if([[column identifier] isEqualToString: @"duration"]) {
    int total = [[info valueForKey: [column identifier]] intValue];
    int min = total / 60;
    int sec = total % 60;
    return [NSString stringWithFormat: @" %02d:%02d", min, sec];
  }

  return [info valueForKey: [column identifier]];
}

- (void) tableView: (NSTableView *)view
      setObjectValue: (id) object
      forTableColumn: (NSTableColumn *) column
                 row: (int) row
{
  [[listForDisplay objectAtIndex: row] takeValue: object
                                          forKey: [column identifier]];
  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBSelectedSongInfoChanged"
           object: [listForDisplay objectAtIndex: row]];
}

- (int)    tableView: (NSTableView *) view
 stateForTableColumn: (NSTableColumn *) column
                 row: (int)row
{
  return [[listForDisplay objectAtIndex: row] checked];
}

- (void)   tableView: (NSTableView *) view
            setState: (int) state
      forTableColumn: (NSTableColumn *) column
                 row: (int) row
{
  [[listForDisplay objectAtIndex: row] setChecked: state];
  [view reloadData];
}

- (void) reloadTableViewData: (NSNotification *) not
{
  [tableView reloadData];
}

- (void) selectRowInTableView: (NSNotification *) not
{
  [tableView selectRow: [[not object] intValue]
             byExtendingSelection: NO];
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [tableView unregisterDraggedTypes];
  RELEASE(selectedPath);
  RELEASE(listForDisplay);
  [super dealloc];
}

- (id) initWithWindowNibName: (NSString *) windowNibName;
{
  /* Get the window */
  self = [super initWithWindowNibName: @"MusicBoxWindow"];
  
  /* Get the bacis controller */
  playlistManager = [PlaylistManager sharedManager];
  actionController = [ActionController sharedController];

  selectedPath = [NSMutableString new];
  listForDisplay = [NSMutableArray new];

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(controlTextDidChange:)
                         name: @"NSControlTextDidChangeNotification"
                                             object: nil];
              
  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(reloadTableViewData:)
                         name: @"MBShouldReloadTableViewData"
                                             object: nil];

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(selectRowInTableView:)
                         name: @"MBSelectedSongInfoShouldChange"
                                             object: nil];

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(reloadBrowserAndTableView:)
                         name: @"MBMusicBoxWindowNeedReloadNotification"
                         object: nil];

  return self;
}

- (void) awakeFromNib
{
  NSRect frame, bounds;
  NSTableColumn *statusCol, *songCol, *artistCol, *albumCol, *durationCol;
  NSButtonCell *buttonCell;

  bounds = [[[self window] contentView] bounds];

  frame = NSMakeRect(5,25,bounds.size.width-10,bounds.size.height-40-25);
  splitView = [[NSSplitView alloc] initWithFrame: frame];
  [splitView setVertical: NO];
  [splitView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [splitView setDelegate: self]; 

  frame = NSMakeRect(0,0,bounds.size.width,((bounds.size.height-25-40)/2));
  browser = [[NSBrowser alloc] initWithFrame: frame];
  [browser setDelegate: self];
  [browser setTarget: self];
  [browser setAction: @selector(browserAction:)];
  [browser setMaxVisibleColumns: 2];
  [browser setAllowsMultipleSelection: NO];
  [browser setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
//  [splitView addSubview: browser];

  /* Set up Button Cell */
  buttonCell = [[NSButtonCell alloc] init];
  [buttonCell setButtonType: NSSwitchButton];
  [buttonCell setImagePosition: NSImageLeft];
  [buttonCell setBezelStyle: NSSmallIconButtonBezelStyle];
  
  /* Tracks subview */
  statusCol = [[NSTableColumn alloc] initWithIdentifier: @"status"];
  [statusCol setEditable: NO];
  [statusCol setMinWidth: 25];
  [statusCol setMaxWidth: 45];
 
  songCol = [[NSTableColumn alloc] initWithIdentifier: @"song"];
  [songCol setEditable: YES];
  [songCol setMinWidth: 50];
  [songCol setDataCell: buttonCell];
  [[songCol headerCell] setStringValue: _(@"Song")];
  RELEASE(buttonCell);

  artistCol = [[NSTableColumn alloc] initWithIdentifier: @"artist"];
  [artistCol setEditable: YES];
  [artistCol setMinWidth: 50];
  [[artistCol headerCell] setStringValue: _(@"Artist")];

  albumCol = [[NSTableColumn alloc] initWithIdentifier: @"album"];
  [albumCol setEditable: YES];
  [albumCol setMinWidth: 50];
  [[albumCol headerCell] setStringValue: _(@"Album")];

  durationCol = [[NSTableColumn alloc] initWithIdentifier: @"duration"];
  [durationCol setEditable: NO];
  [durationCol setMinWidth: 50];
  [[durationCol headerCell] setStringValue: _(@"Duration")];

  frame = NSMakeRect(0,0,bounds.size.width,((bounds.size.height-40-25)/2));
  scrollView = [[NSScrollView alloc] initWithFrame: frame];
  [scrollView setHasHorizontalScroller:YES];
  [scrollView setHasVerticalScroller:YES];
  [scrollView setBorderType: NSBezelBorder];
  [scrollView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];

  frame = NSMakeRect(0,0,bounds.size.width,((bounds.size.height-40-25)/2)-1);
  tableView = [[ExtendedTableView alloc] initWithFrame: frame];
  [tableView setDrawsGrid: NO];
  [tableView addTableColumn: statusCol];
  [tableView addTableColumn: songCol];
  [tableView addTableColumn: artistCol];
  [tableView addTableColumn: albumCol];
  [tableView addTableColumn: durationCol];
  [tableView setAutoresizesAllColumnsToFit: YES];
  [tableView setAllowsColumnSelection: NO];
  [tableView setAllowsMultipleSelection: YES];
  [tableView setAllowsEmptySelection: YES];
  [tableView setTarget: self];
  [tableView setDataSource: self];
  [tableView setDelegate: self];
  [tableView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [tableView setDoubleAction: @selector(doubleClickOnTable:)];
  [tableView setButtonColumnIdentifier: @"song"];
  [tableView registerForDraggedTypes: [NSArray arrayWithObjects: NSGeneralPboardType, nil]];

  RELEASE(statusCol);
  RELEASE(songCol);
  RELEASE(artistCol);
  RELEASE(albumCol);
  RELEASE(durationCol);

  [scrollView setDocumentView: tableView];
  RELEASE(tableView);

  [splitView addSubview: scrollView];
  [splitView adjustSubviews];

  [[[self window] contentView] addSubview: splitView];
  RELEASE(scrollView);
  RELEASE(splitView);

  [[self window] makeFirstResponder: searchField];
  [[self window] setFrameAutosaveName: @"MusicBoxWindow"];
  [[self window] setFrameUsingName: @"MusicBoxWindow"];
  [[self window] setMinSize: NSMakeSize(500, 300)];

  frameOfBrowser = [browser bounds];
  frameOfScrollView = [scrollView bounds];

  /* Get the outlet and redirect action if necessary */
  [repeatButton setTarget: actionController];
  [repeatButton setAction: @selector(setRepeat:)];
  [shuffleButton setTarget: actionController];
  [shuffleButton setAction: @selector(setShuffle:)];
}

- (void) updateStateBar
{
  NSMutableString *status;
  int i, totalTime = 0, totalSize = 0;
  int count = [listForDisplay count];
  int hour, minute, second;
  NSString *songUnit = _(@"song");
  NSString *hourUnit = _(@"hour");
  NSString *minuteUnit = _(@"minute");
  NSString *secondUnit = _(@"second");

  // Calculate the status

  for (i = 0; i < count; i++) {
    totalTime += [[(SongInfo *)[listForDisplay objectAtIndex: i] duration] intValue];
    totalSize += [[(SongInfo *)[listForDisplay objectAtIndex: i] size] intValue];
  }

  if (count > 1)
    songUnit = _(@"songs");;

  status = [NSMutableString stringWithFormat: @"%d %@,", count, songUnit];

  hour = (totalTime / 3600);
  if (hour > 0) 
    {
      if (hour > 1)
        hourUnit = _(@"hours");
      [status appendFormat: @" %d %@", hour, hourUnit]; 
    }

  minute = ((totalTime % 3600) / 60);
  if ((hour > 0) || (minute > 0))
    {
      if (minute > 1)
        minuteUnit = _(@"minutes");
      [status appendFormat: @" %d %@", minute, minuteUnit];
    }

  second = (totalTime % 60);
  if (hour == 0)
    {
      if (second > 1)
        secondUnit = _(@"seconds");
      [status appendFormat: @" %d %@", second, secondUnit];
    }

  [status appendFormat: @", %.2f %@", 
                      (float)(totalSize/(1024.0*1024)), _(@"MB")];

  [statusLabel setStringValue: status];
}

@end
