/*
**  PlayerView.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "PlayerView.h"
#include "PlaylistManager.h"
#include "ActionController.h"
#include <AppKit/AppKit.h>

@implementation PlayerView

- (void) prev: (id) sender
{
  [actionController prev: sender];
}

- (void) play: (id) sender
{
  [actionController play: sender];
}

- (void) pause: (id) sender
{
  [actionController pause: sender];
}

- (void) next: (id) sender
{
  [actionController next: sender];
}

- (void) controlStatusChanged: (NSNotification *) not
{
  if ([[not object] isEqualToString: @"Playing"])
    {
      [play setImage: [NSImage imageNamed: @"pause.tiff"]];
      [play setAction: @selector(pause:)];
      [play setNeedsDisplay: YES];
    }
  else if ([[not object] isEqualToString: @"Encoding"])
    {
      [play setEnabled: NO];
      [prev setEnabled: NO];
      [next setEnabled: NO];
    }
  else if ([[not object] isEqualToString: @"EncodingStopped"])
    {
      [play setEnabled: YES];
      [prev setEnabled: YES];
      [next setEnabled: YES];
    }
  else
    {
      [play setImage: [NSImage imageNamed: @"play.tiff"]];
      [play setAction: @selector(play:)];
      [play setNeedsDisplay: YES];
    }
}

- (void) setLED: (id) aled
{
  NSRect frame = [self bounds];
  ASSIGN(led, aled);
  [led setFrame: NSMakeRect(140, frame.size.height-65, frame.size.width-145, 60)];
  [led setAutoresizingMask: NSViewWidthSizable | NSViewMinYMargin];
  [self addSubview: led];
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [super dealloc];
}

- (id) initWithFrame: (NSRect) rect
{
  NSRect frame;

  self = [super initWithFrame: rect];

  playlistManager = [PlaylistManager sharedManager];
  actionController = [ActionController sharedController];
  
  frame = NSMakeRect(5, rect.size.height-45, 40, 40);
  prev = [[NSButton alloc] initWithFrame: frame];
  [prev setButtonType: NSMomentaryPushButton];
  [prev setImagePosition: NSImageOnly];
  [prev setImage: [NSImage imageNamed: @"prev.tiff"]];
  [prev setAutoresizingMask: NSViewMinYMargin];
  [prev setTarget: self];
  [prev setAction: @selector(prev:)];
  [self addSubview: prev];
  RELEASE(prev);
  
  frame = NSMakeRect(50, rect.size.height-45 , 40, 40);
  play = [[NSButton alloc] initWithFrame: frame];
  [play setButtonType: NSMomentaryPushButton];
  [play setImagePosition: NSImageOnly];
  [play setImage: [NSImage imageNamed: @"play.tiff"]];
  [play setAutoresizingMask: NSViewMinYMargin];
  [play setTarget: self];
  [play setAction: @selector(play:)];
  [self addSubview: play];
  RELEASE(play);

  frame = NSMakeRect(95, rect.size.height-45, 40, 40);
  next = [[NSButton alloc] initWithFrame: frame];
  [next setButtonType: NSMomentaryPushButton];
  [next setImagePosition: NSImageOnly];
  [next setImage: [NSImage imageNamed: @"next.tiff"]];
  [next setAutoresizingMask: NSViewMinYMargin];
  [next setTarget: self];
  [next setAction: @selector(next:)];
  [self addSubview: next];
  RELEASE(next);

  frame = NSMakeRect(5, rect.size.height-64, 130, 12);
  slider = [[NSSlider alloc] initWithFrame: frame];
  [slider setMinValue: 0];
  [slider setMaxValue: 1];
  [slider setFloatValue: [actionController volume]];
  [slider setAutoresizingMask: NSViewMinYMargin];
  [slider setTarget: actionController];
  [slider setAction: @selector(sliderAction:)];
  [self addSubview: slider];
  RELEASE(slider);

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(controlStatusChanged:)
                         name: @"MBControlStatusChanged"
                         object: nil];

  return self;
}

- (NSButton *) prevButton
{
  return prev;
}

- (NSButton *) playButton
{
  return play;
}

- (NSButton *) nextButton
{
  return next;
}

- (NSSlider *) slider
{
  return slider;
}

@end
