/*
**  PlaylistManager.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _Musicbox_MusicBox_H_PlaylistManager
#define _Musicbox_MusicBox_H_PlaylistManager

#include <Foundation/NSObject.h>

@class AudioList; 
@class SongInfo;
@class MusicBoxWindowController;
@class NSMutableArray;

@interface PlaylistManager: NSObject
{
  MusicBoxWindowController *mainWindowController;
  NSMutableArray *sourceList;
  int selectedPlaylistForPlay;
  int selectedPlaylist; // Selected play list
  int selectedSongForPlay; // Selected song
}

/* The file name is in user default PlaylistFile */

+ (PlaylistManager *) sharedManager;

- (id) initFromFile;
- (void) writeToFile;

- (void) setMainWindowController: (MusicBoxWindowController *) controller;
- (MusicBoxWindowController *) mainWindowController;

  /* Sync the displayed playlist to listForPlay */
- (void) syncPlaylist;

- (NSArray *) allPlaylist;
- (NSArray *) allNamesOfPlaylists;
- (unsigned) count;

/* These three methods will update the selected automatically */
- (void) addPlaylist: (AudioList *) playlist;
- (void) movePlaylist: (int) fromIndex to: (int) toIndex;
- (void) removePlaylist: (int) index;

- (AudioList *) playlistAt: (int) index;
- (AudioList *) mainLibrary;

/* Corresponds to playlist window */
- (AudioList *) selectedPlaylist;
- (int) indexOfSelectedPlaylist;
- (void) selectPlaylist: (int) index;
- (void) removeSelectedPlaylist;

/* These methods should (hopely) retain the array */
- (void) selectPlaylistForPlay: (int) index;
- (AudioList *) selectedPlaylistForPlay;
- (NSArray *) soundListForPlay; /* [AudioList soundList] */

/* Short-cut method to get list for play */
- (SongInfo *) songInfoForPlay: (int) index;

/* Methods for selection, correspond to the one in main window */
- (void) selectSongForPlay: (int) index;
- (int) indexOfSelectedSongForPlay;

/* Short-cut method for selection */
- (SongInfo *) selectedSongInfoForPlay;

@end

#endif /* _Musicbox_MusicBox_H_PlaylistManager */
