/*
**  PlaylistWindowController.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "PlaylistWindowController.h"
#include <MusicBox/ModuleLoader.h>
#include <MusicBox/AudioList.h>
#include "IconTableView.h"
#include "PlaylistManager.h"
#include "SourceEditorWindowController.h"
#include "MusicBoxWindowController.h"

@implementation PlaylistWindowController

- (void) deletePlaylist: (id) sender
{
/*
  if (useMultipleWindows == YES)
    RELEASE([self windowOfName: [[playlistManager selectedPlaylist] name]]);
*/

  [playlistManager removeSelectedPlaylist]; 
  [tableView reloadData];

  if ([playlistManager indexOfSelectedPlaylist] > -1)
    [tableView selectRow: [playlistManager indexOfSelectedPlaylist]
               byExtendingSelection: NO];
}

- (void) addPlaylist: (id) sender
{
  SourceEditorWindowController *sourceEditorController;
  NSAutoreleasePool *pool;
  int result;
 
  pool = [[NSAutoreleasePool alloc] init];

  sourceEditorController = [[SourceEditorWindowController alloc] initWithWindowNibName: @"SourceEditorWindow"];

  [sourceEditorController setExistingNames: [playlistManager allNamesOfPlaylists]];

  result = [NSApp runModalForWindow: [sourceEditorController window]];

  if (result == NSRunStoppedResponse)
    {
      AudioList *newList = [sourceEditorController newPlaylist];
      [playlistManager addPlaylist: newList];
    }

  [tableView reloadData];

  if ([playlistManager indexOfSelectedPlaylist] > -1)
    [tableView selectRow: [playlistManager indexOfSelectedPlaylist]
               byExtendingSelection: NO];

  RELEASE(sourceEditorController);
  RELEASE(pool);
}

- (void) editPlaylist: (id)sender
{
  SourceEditorWindowController *sourceEditorController;
//  MusicBoxWindow *window;
  NSAutoreleasePool *pool;
  NSMutableArray *array;
  int result;

  pool = [[NSAutoreleasePool alloc] init];

  sourceEditorController = [[SourceEditorWindowController alloc] initWithWindowNibName: @"SourceEditorWindow"];

  [sourceEditorController setPlaylist: [playlistManager selectedPlaylist]];

/*
  if (useMultipleWindows == YES)
    {
      window = [self windowOfName: [[playlistManager selectedPlaylist] name]];
    }
*/

  array = [NSMutableArray arrayWithArray: [playlistManager allNamesOfPlaylists]];
  [array removeObject: [[playlistManager selectedPlaylist] name]];
  [sourceEditorController setExistingNames: array];

  result = [NSApp runModalForWindow: [sourceEditorController window]];

  if (result == NSRunStoppedResponse)
    {
      AudioList *newList = [sourceEditorController newPlaylist];
      [[playlistManager selectedPlaylist] setName: [[newList name] copy]];
      [[playlistManager selectedPlaylist] setProperty: [[newList property] copy]];
/*
      if ((useMultipleWindows) && (window))
        {
          // FIXME: Window title doesn't change
          [window setTitle: [[playlistManager selectedPlaylist] name]];
        }
*/
    }

  [tableView reloadData];
  [tableView selectRow: [playlistManager indexOfSelectedPlaylist]
             byExtendingSelection: NO];

  RELEASE(sourceEditorController);
  RELEASE(pool);

  [[NSNotificationCenter defaultCenter]
       postNotificationName: @"MBMusicBoxWindowNeedReloadNotification"
       object: nil];
}

// NSTableView delegate

- (void) tableViewSelectionDidChange: (NSNotification *) not
{
  NSRect frame = [[self window] frame];

  [playlistManager selectPlaylist: [tableView selectedRow]];
  [tableView scrollRowToVisible: [tableView selectedRow]];

  if (useMultipleWindows == NO) 
    {
      [[playlistManager mainWindowController] setAudioList: [playlistManager selectedPlaylist]];
      [[[playlistManager mainWindowController] window] setTitle: [[playlistManager selectedPlaylist] name]];
    }

  if (additionalButton)
    [additionalButton removeFromSuperview];

  additionalButton = [[playlistManager selectedPlaylist] additionalButtonOnPlaylistWindow];
  if (additionalButton)
    {
      [additionalButton setFrame: NSMakeRect(frame.size.width-35, frame.size.height-65, 30, 30)];
      [additionalButton setAutoresizingMask: NSViewMinXMargin | NSViewMinYMargin];
      [[[self window] contentView] addSubview: additionalButton];
    }
}

/*
- (MusicBoxWindow *) windowOfName: (NSString *) title
{
  MusicBoxWindow *window;
  NSArray *windowArray = [NSApp windows];
  NSString *name = [[playlistManager selectedPlaylist] name];
  BOOL isWindowExisting = NO;
  int i = [windowArray count] - 1;
  
  for (; i > 0; i--)
    {
      window = [windowArray objectAtIndex: i];
      if ([[window title] isEqualToString: title])
        {
          if ([window isMemberOfClass: [MusicBoxWindow class]])
            {
              return window;
            }
        }
    }
  return nil;
}
*/

- (void) doubleClickOnTable: (id) sender
{
  MusicBoxWindowController *controller;
//  MusicBoxWindow *window;
  NSString *name;

  if (useMultipleWindows == NO)
    {
      [[[playlistManager mainWindowController] window] makeKeyAndOrderFront: self];
    }
  else
    {
/*
      name  = [[playlistManager selectedPlaylist] name];

      window = [self windowOfName: name];
      if (window)
        {
          [window makeKeyAndOrderFront: self];
        }
      else
        {
          controller = [[MusicBoxWindowController alloc] initWithWindowNibName: @"MusicBoxWindow"];
          [controller setAudioList: [playlistManager selectedPlaylist]];
// FIXME: Should release window to save memory ?
//          [[controller window] setReleasedWhenClosed: YES];
          [[controller window] setTitle: name];
          [[controller window] makeKeyAndOrderFront: self];
        }
*/
    }
}

- (BOOL) tableView: (NSTableView*)view
        acceptDrop: (id <NSDraggingInfo>)info
               row: (int)row
     dropOperation: (NSTableViewDropOperation)operation
{
  NSPasteboard *pboard;
  NSData *data;

  NSLog(@"row %d", row);

  pboard = [info draggingPasteboard];
  data = [pboard dataForType: @"NSGeneralPboardType"];
  
  if (nil == [info draggingSource]) // From other application
    {
      return NO;
    }
  else if (tableView == [info draggingSource])
    {
      NSNumber *index;

      index = [NSUnarchiver unarchiveObjectWithData: data];
      NSLog(@"Drag from %@ into %d", index, row);
      [playlistManager movePlaylist: [index intValue]
                                 to: row];
      [tableView reloadData];
      
      return YES;
    }
  else // From musicbox window
    {
      NSArray *array;
      BOOL result;

      if (row > [playlistManager count]-1)
        return NO;

      array = [NSUnarchiver unarchiveObjectWithData: data];
      result =  [[playlistManager playlistAt: row] addObjects: array];
      [tableView reloadData];

      return result;
    }
}

- (NSDragOperation) tableView: (NSTableView*)view
                 validateDrop: (id <NSDraggingInfo>)info
                  proposedRow: (int)row
        proposedDropOperation: (NSTableViewDropOperation)operation
{
  if (row > [playlistManager count] - 1)
    return NSDragOperationNone;

  if (nil == [info draggingSource]) // From other application 
    {
      [view setDropRow: row dropOperation: NSTableViewDropOn];
      return NSDragOperationNone;
    }
  else if (tableView == [info draggingSource])
    {
      [view setDropRow: row dropOperation: NSTableViewDropAbove];
      return NSDragOperationGeneric;
    }
  else // Come from musicbox window 
    {
      [view setDropRow: row dropOperation: NSTableViewDropOn];

      if ([[playlistManager playlistAt: row] canDropSoundList])
        return NSDragOperationCopy;
      else
        return NSDragOperationNone;
    }
}

- (BOOL) tableView: (NSTableView *) view writeRows: (NSArray *)rows
         toPasteboard: (NSPasteboard *) pboard
{
  NSData *data;

  data = [NSArchiver archivedDataWithRootObject: [rows objectAtIndex: 0]];

  [pboard declareTypes: [NSArray arrayWithObject: @"NSGeneralPboardType"]
                 owner: nil];
  [pboard setData: data forType: @"NSGeneralPboardType"];

  return YES;
}

- (int) numberOfRowsInTableView: (NSTableView *) aView
{
  return [playlistManager count];
}

- (id) tableView: (NSTableView *) view
           objectValueForTableColumn: (NSTableColumn *) column
           row: (int) row
{
  AudioList *list = [playlistManager playlistAt: row];

  if ([[column identifier] isEqualToString: @"source"])
    return [list name];
  else if ([[column identifier] isEqualToString: @"songs"])
    return [NSNumber numberWithInt: [list count]];
  else
    return @"";
}

- (id) tableView: (NSTableView *) view
           iconForTableColumn: (NSTableColumn *) column
           row: (int) row
{
  if ([[column identifier] isEqualToString: @"source"])
    return [[playlistManager playlistAt: row] icon];
  else
    return nil;
}

- (void) reloadTableViewData: (NSNotification *) not
{
  [tableView reloadData]; 
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [tableView unregisterDraggedTypes];
  [super dealloc];
}

- (id) initWithWindowNibName: (NSString *) windowNibName;
{
  /* Get the bacis controller */
  playlistManager = [PlaylistManager sharedManager];

  /* Get the window */
  self = [super initWithWindowNibName: @"PlaylistWindow"];

  return self;
}

- (void) awakeFromNib
{
  NSRect frame, bounds;
  NSTableColumn *sourceCol, *songsCol;
  NSScrollView *scrollView;

  /* Set up buttons */
  [addButton setImagePosition: NSImageOnly];
  [addButton setImage: [NSImage imageNamed: @"stock_add_24.png"]];
  [editButton setImagePosition: NSImageOnly];
  [editButton setImage: [NSImage imageNamed: @"stock_properties_24.png"]];
  [deleteButton setImagePosition: NSImageOnly];
  [deleteButton setImage: [NSImage imageNamed: @"stock_remove_24.png"]];

  /* Set up columns */
  sourceCol = [[NSTableColumn alloc] initWithIdentifier: @"source"];
  [sourceCol setEditable: NO];
  [sourceCol setMinWidth: 50];
  [[sourceCol headerCell] setStringValue: _(@"Source")];
 
  songsCol = [[NSTableColumn alloc] initWithIdentifier: @"songs"];
  [songsCol setEditable: NO];
  [songsCol setMinWidth: 25];
  [songsCol setMaxWidth: 50];
  [[songsCol headerCell] setStringValue: _(@"Songs")];

  bounds = [[[self window] contentView] bounds];

  frame = NSMakeRect(0,0,bounds.size.width, (bounds.size.height-40));
  scrollView = [[NSScrollView alloc] initWithFrame: frame];
  [scrollView setHasHorizontalScroller:NO];
  [scrollView setHasVerticalScroller:YES];
  [scrollView setBorderType: NSBezelBorder];
  [scrollView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];

  frame = NSMakeRect(0,0,bounds.size.width,(bounds.size.height-40));
  tableView = [[IconTableView alloc] initWithFrame: frame];
  [tableView setDrawsGrid: NO];
  [tableView addTableColumn: sourceCol];
  [tableView addTableColumn: songsCol];
  [tableView setAutoresizesAllColumnsToFit: YES];
  [tableView setAllowsColumnSelection: NO];
  [tableView setAllowsMultipleSelection: NO];
  [tableView setAllowsEmptySelection: NO];
  [tableView setTarget: self];
  [tableView setDataSource: self];
  [tableView setDelegate: self];
  [tableView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [tableView setDoubleAction: @selector(doubleClickOnTable:)];
  [tableView setVerticalMotionCanBeginDrag: YES];
  [tableView setVerticalMotionCanBeginDrag: YES];
  RELEASE(sourceCol);
  RELEASE(songsCol);

  [scrollView setDocumentView: tableView];
  RELEASE(tableView);

  [[[self window] contentView] addSubview: scrollView];
  RELEASE(scrollView);

  [[self window] setFrameAutosaveName: @"PlaylistWindow"];
  [[self window] setFrameUsingName: @"PlaylistWindow"];
  [[self window] setMinSize: NSMakeSize(100, 150)];

  [tableView registerForDraggedTypes: [NSArray arrayWithObjects: NSGeneralPboardType, nil]];

  if ([playlistManager count] > 0)
    {
        [playlistManager selectPlaylist: 0];
        [tableView selectRow: 0
                   byExtendingSelection: NO];
    }

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(reloadTableViewData:)
                         name: @"MBShouldReloadPlaylistNotification"
                                             object: nil];

  useMultipleWindows = [[NSUserDefaults standardUserDefaults] boolForKey: @"UseMultipleWindows"];
}

@end
