/*
**  FreeDBViewController.m
**
**  Copyright (c) 2002
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "FreeDBViewController.h"
#include "FreeDBView.h"
#include <Cddb/Cddb.h>
#include <MusicBox/ModuleLoader.h>
#include <AppKit/AppKit.h>

@implementation FreeDBViewController

- (id) init
{
  ModuleLoader *loader = [ModuleLoader defaultLoader];

  self = [super init];

  cddbClass = [loader classNameOf: @"Cddb.bundle"];

  if (cddbClass) 
    {
      view = [[FreeDBView alloc] initWithFrame: NSMakeRect(0, 0, 400, 200)];

      freedbField = [view freedbField];
      tableView = [view tableView];
      [tableView setDelegate: self];
      [tableView setTarget: self];
      [tableView setDataSource: self];
      button = [view button];
      [button setTarget: self];
      [button setAction: @selector(listOfSite:)];
    }

  siteList = [NSMutableArray new];

  [self initializeFromDefaults];

  return self;
}

- (void) dealloc
{
  RELEASE(view);
  RELEASE(siteList);
  [super dealloc];
}

- (NSImage *) image
{
/*
  NSBundle *aBundle;
  
  aBundle = [NSBundle bundleForClass: [self class]];
  
  return AUTORELEASE([[NSImage alloc] initWithContentsOfFile:
		[aBundle pathForResource: @"fonts" ofType: @"tiff"]]);
*/
  return nil;
}

- (NSString *) name
{
  return _(@"Freedb");
}

- (NSView *) view
{
  return view;
}

- (BOOL) hasChangesPending
{
  return YES;
}

// Data Source
- (void) tableViewSelectionDidChange: (NSNotification *) not
{
  [freedbField setStringValue: [[siteList objectAtIndex: [[not object] selectedRow]]
                 objectForKey: @"site"]];
}

- (int) numberOfRowsInTableView: (NSTableView *) aView
{
  return [siteList count];
}

- (id) tableView: (NSTableView *) aView
           objectValueForTableColumn: (NSTableColumn *) aColumn
           row: (int) row
{
  return [[siteList objectAtIndex: row] objectForKey: [aColumn identifier]];
}

- (void) listOfSites: (id) sender
{
  Cddb *cddb;
  NSArray *siteArray = nil;
  int i;

  cddb = [cddbClass new];
   
  [siteList removeAllObjects];
  // Always get list from main site (www.freedb.org).
  [cddb setDefaultSite: @"http://www.freedb.org:80/~cddb/cddb.cgi"];

  if([cddb connect]) {
    siteArray = [cddb sites];
    [cddb disconnect];
  }
  
  if (siteArray == nil) {
    NSRunAlertPanel(@"Can't get list from internet",
                    @"Can't get the list of public freedb sites.\nMake sure you have internet connection and the connection to the official freedb site (www.freedb.org) works.\n",
                    @"O.K.",
                    nil,
                    nil);
    return;
  }

  for(i = 0; i < [siteArray count]; i++) {
    NSMutableDictionary *dict = [NSMutableDictionary new];
    NSString *site;
    if([[[siteArray objectAtIndex: i] objectForKey: @"protocol"] 
                       isEqualToString: @"http"]) 
      {
        site = [NSString stringWithFormat: @"%@://%@:%@%@",
                       [[siteArray objectAtIndex: i] objectForKey: @"protocol"],
                       [[siteArray objectAtIndex: i] objectForKey: @"site"],
                       [[siteArray objectAtIndex: i] objectForKey: @"port"],
                       [[siteArray objectAtIndex: i] objectForKey: @"address"]];
      }
    else
      {
        site = [NSString stringWithFormat: @"%@://%@:%@",
                       [[siteArray objectAtIndex: i] objectForKey: @"protocol"],
                       [[siteArray objectAtIndex: i] objectForKey: @"site"],
                       [[siteArray objectAtIndex: i] objectForKey: @"port"]];
      }
    [dict setObject: site forKey: @"site"];
    [dict setObject: [[siteArray objectAtIndex: i] objectForKey: @"description"]
                     forKey: @"location"];
    [siteList addObject: dict];
  }
  [tableView reloadData];
}

- (void) initializeFromDefaults
{
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"FreedbSite"] )
    {
      [freedbField setStringValue: [[NSUserDefaults standardUserDefaults] 
				      stringForKey: @"FreedbSite"]];
    }
}

- (void) saveChanges
{
  [[NSUserDefaults standardUserDefaults] setObject: [freedbField stringValue]
                                         forKey: @"FreedbSite"];
}

@end
