/*
**  PreferenceWindowController.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "PreferenceWindowController.h"
#include <MusicBox/PreferenceModule.h>
#include <MusicBox/ModuleLoader.h>
#include <AppKit/AppKit.h>


@implementation PreferenceWindowController

- (id) initWithWindowNibName: (NSString *) windowNibName
{
  self = [super initWithWindowNibName: windowNibName];

  allModules = [NSMutableArray new];

  return self;
}

- (void) dealloc
{
  RELEASE(allModules);
  [super dealloc];
}

- (void) awakeFromNib
{
//  NSDictionary *allPreferences;
  NSButtonCell *cell;
  NSScrollView *scrollView;
  NSRect frame, bounds, boxFrame;
  NSArray *allPrefs;
  ModuleLoader *loader = [ModuleLoader defaultLoader];
  id <PreferenceModule> module;
  int i;

  // We copy the current preferences to a volatile domain
/*
  allPreferences = [NSDictionary dictionaryWithDictionary: [[NSUserDefaults standardUserDefaults] dictionaryRepresentation]];
  // FIXME - This cause a segfault on OS X when reloading a 2nd time the preferences panel
#ifndef MACOSX
  [[NSUserDefaults standardUserDefaults] removeVolatileDomainForName: @"PREFERENCES"];
#endif
  [[NSUserDefaults standardUserDefaults] setVolatileDomain: allPreferences forName: @"PREFERENCES"];
*/

  // We initialize our matrix with the standard modules
  allPrefs = [loader allModulesNames: @"pref"];
  NSLog(@"allPrefs %@", allPrefs);
  for (i = 0; i < [allPrefs count]; i++)
    {
      Class moduleClass = [[ModuleLoader defaultLoader] classNameOf: [allPrefs objectAtIndex: i]];

      if (!moduleClass)
        {
          NSLog(@"Can't load module %@", [allPrefs objectAtIndex: i]);
          continue;
        }

      module = [[moduleClass alloc] init]; 

      if (!module)
        {
          NSLog(@"Can't get instance %@", moduleClass);
          continue;
        }
 
      [allModules addObject: module];
      RELEASE(module);
    }

  bounds = [[[self window] contentView] bounds];
  boxFrame = [box frame];

  cell = [[NSButtonCell alloc] init];
  [cell setButtonType: NSPushOnPushOffButton];
//  [cell setImagePosition: NSImageAbove];
  [cell setImagePosition: NSNoImage];

  frame = NSMakeRect(boxFrame.origin.x, boxFrame.origin.y+boxFrame.size.height,
                     boxFrame.size.width,
                     bounds.size.height-boxFrame.origin.y-boxFrame.size.height-5
);

  
  NSLog(@"frame %@", NSStringFromRect(frame));
  matrix = [[NSMatrix alloc] initWithFrame: frame
                             mode: NSRadioModeMatrix
                             prototype: cell
                             numberOfRows: 1
                             numberOfColumns: [allModules count]];
  [matrix setTarget: self];
  [matrix setCellSize: NSMakeSize(frame.size.height,frame.size.height)];
  [matrix setAction: @selector(handleCellAction:)];
  
  scrollView = [[NSScrollView alloc] initWithFrame: frame];
  [scrollView setHasHorizontalScroller: YES];
  [scrollView setHasVerticalScroller: NO];
  [scrollView setDocumentView: matrix];  
  [[[self window] contentView] addSubview: scrollView];
  RELEASE(cell);
  RELEASE(matrix);
  RELEASE(scrollView);
  
  for (i = 0; i < [allModules count]; i++)
    {
      /* In case the module failed to load */
      cell = [matrix cellAtRow: 0 column: i];
      module = [allModules objectAtIndex: i];
      [cell setTag: i];
      [cell setTitle: [module name]];
      [cell setImage: [module image]];
    }  

  if ([allModules count])
    {
      [matrix selectCellWithTag: 0];
      [self handleCellAction: matrix];
    }

  [[self window] setFrameAutosaveName: @"PreferenceWindow"];
  [[self window] setFrameUsingName: @"PreferenceWindow"];
}

- (void) handleCellAction: (id) sender
{  
  id <PreferenceModule> module;
  
  module = [allModules objectAtIndex: [[sender selectedCell] tag]];

  if ( module )
    {
      if ( [box contentView] != [module view] )
        {
          [[module view] setFrame: [[box contentView] frame]];
          [box setContentView: [module view]];
          [box setTitle: [module name]];
        }
    }
  else
    {
      NSLog(@"Unable to load the %@ bundle.", [[matrix selectedCell] title]);
    }
}

// action methods
- (void) revertAction: (id) sender
{
  [NSApp stopModal];
  [self close];
}

- (void) applyAction: (id) sender
{
  NSArray *allNames;
  id <PreferenceModule> module;
  int i;

  for (i = 0; i < [allNames count]; i++)
    {
      module = [allModules objectAtIndex: i];
      [module saveChanges];
    }
  
  [[NSUserDefaults standardUserDefaults] synchronize];

  [NSApp stopModal];
  [self close];
}

- (void) windowWillClose: (NSNotification *) not
{
  [NSApp stopModal];
}

@end
