/*
**  SourceEditorWindowController.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "SourceEditorWindowController.h"
#include <MusicBox/AudioList.h>
#include <MusicBox/ModuleLoader.h>
#include <AppKit/AppKit.h>

@implementation SourceEditorWindowController

- (void) setExistingNames: (NSArray *) array
{
  ASSIGN(existingNames, array);
}

- (void) setPlaylist: (AudioList *) thePlaylist
{
  ASSIGN(playlist, thePlaylist);
}

- (void) dealloc
{
  RELEASE(allClasses);
  RELEASE(existingNames);
  RELEASE(playlist);
  RELEASE(playlistView);
  [super dealloc];
}

// action methods

- (void) okAction: (id) sender
{  
  NSString *name = [[playlistView property] objectForKey: @"name"];
  if ((name == nil) || ([[name stringByTrimmingSpaces] isEqualToString: @""]))
    {
      NSRunAlertPanel(_(@"Playlist Error"),
                      _(@"You have to assign the name of playlist"),
                      _(@"OK"),
                      nil,
                      nil);
    }
  else if ((existingNames) && ([existingNames containsObject: name]))
    {
      NSRunAlertPanel(_(@"Playlist Error"),
                      _(@"A playlist already has the same name.\nPlease assign another one"),
                      _(@"OK"),
                      nil,
                      nil);
    }
  else
    {
      [NSApp stopModal];
      [self close];
    }
}

- (void) cancelAction: (id) sender
{
  [NSApp abortModal];
  [self close];
}

- (void) typeButtonAction: (id) sender
{
  ASSIGN(playlistView,
        [[allClasses objectAtIndex: [sender indexOfSelectedItem]] settingView]);

  [box setContentView: playlistView];
}

- (AudioList *) newPlaylist;
{
  AudioList *newSource = nil;
  Class audioClass = [allClasses objectAtIndex: [typeButton indexOfSelectedItem]];
  if ([[audioClass description] isEqualToString: @"AudioList"] == NO)
    {
      audioClass = [[ModuleLoader defaultLoader] classNameOf: [[audioClass description] stringByAppendingPathExtension: @"audio"]];
    }
 
  newSource = [[audioClass alloc] initWithOwner: [NSApp delegate]];
  [newSource setProperty: [playlistView property]];

  return AUTORELEASE(newSource);
}

- (void) awakeFromNib
{
  ModuleLoader *loader = [ModuleLoader defaultLoader];
  int i;

  [okButton setKeyEquivalent: @"\r"];
  [okButton setImagePosition: NSImageRight];
  [okButton setImage: [NSImage imageNamed: @"common_ret"]];
  [okButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];

  allClasses = [NSMutableArray new];
  [typeButton removeAllItems];

  if (playlist) /* Edit Playlist */
    {
      NSMutableDictionary *property;

      [typeButton addItemWithTitle: [[playlist class] classType]];
      [typeButton setEnabled: NO];
      [allClasses addObject: [playlist class]];
      ASSIGN(playlistView, [[playlist class] settingView]);

      /* the property may not have the name */
      property = [NSMutableDictionary dictionaryWithDictionary: 
                                                [playlist property]];   
      [property setObject: [playlist name] forKey: @"name"];
      [playlistView setProperty: property];
    }
  else /* Add Playlist */
    { 
      NSArray *allModules;

      [allClasses addObject: [AudioList class]]; 
      [typeButton addItemWithTitle: [[allClasses objectAtIndex: 0] classType]];

      allModules = [loader allModulesNames: @"audio"];

      for (i = 0; i < [allModules count]; i++)
        {
          Class audioClass = [loader classNameOf: [allModules objectAtIndex: i]];
          if ([[audioClass classType] isEqualToString: @"Audio CD"])
            continue;
          if ([[audioClass classType] isEqualToString: @"Library"])
            continue;
          [allClasses addObject: audioClass];
          [typeButton addItemWithTitle: [audioClass classType]];
        }

      [typeButton selectItemAtIndex: 0];

      ASSIGN(playlistView, [[allClasses objectAtIndex: 0] settingView]);
    }
  [box setContentView: playlistView];
}

@end
