/*
**  TagEditorPanelController.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "TagEditorPanelController.h"
#include <ShengGuang/SongInfo.h>
#include <ShengGuang/OGGEditor.h>
#include <AppKit/AppKit.h>

@implementation TagEditorPanelController

- (void) setSongInfo: (NSNotification *) not
{
  OGGEditor *editor;
  NSNumber *bitrate;

  info = [not object];
  if (info)
    {
      editor = [[OGGEditor alloc] initWithFile: [info filename]];
      if (editor)
        {
          [channelsField setStringValue: [NSString stringWithFormat: @"Channel(s): %@", [editor channels]]];
          [rateField setStringValue: [NSString stringWithFormat: @"Rate: %@ KHz", [editor rate]]];
          bitrate = [editor nominalOfBitrate];
          if (bitrate)
            {
              if (([bitrate isEqualToNumber: [editor upperOfBitrate]]) 
                  && ([bitrate isEqualToNumber: [editor lowerOfBitrate]]))
                {
                  /* fixed rate bitstream */
                  [bitrateField setStringValue: [NSString stringWithFormat: @"Bitrate: %@ kb", bitrate]];
                }
              else
                {
                  /* variable bit rate */
                  [bitrateField setStringValue: [NSString stringWithFormat: @"Bitrate: %@ kb (VBR)", bitrate]];
                }
            }
          else
            {
              [bitrateField setStringValue: nil];
            }
          RELEASE(editor);
          [writeButton setEnabled: YES];
        }
      else
        {
          [channelsField setStringValue: nil];
          [rateField setStringValue: nil];
          [bitrateField setStringValue: nil];
          [writeButton setEnabled: NO];
        }
      [songField setStringValue: [info song]];
      [artistField setStringValue: [info artist]];
      [albumField setStringValue: [info album]];
      [lengthField setStringValue: [NSString stringWithFormat: @"Duration: %02d:%02d",
                                       [[info duration] intValue] / 60,
                                       [[info duration] intValue] % 60]];
      [okButton setEnabled: YES];
      [applyButton setEnabled: YES];
    }
  else
    {
      [songField setStringValue: nil];
      [songField setEditable: NO];
      [artistField setStringValue: nil];
      [artistField setEditable: NO];
      [albumField setStringValue: nil];
      [albumField setEditable: NO];
      [lengthField setStringValue: nil];
      [channelsField setStringValue: nil];
      [rateField setStringValue: nil];
      [bitrateField setStringValue: nil];
      [writeButton setEnabled: NO];
      [okButton setEnabled: NO];
      [applyButton setEnabled: NO];
    }
}

- (void) awakeFromNib
{
  // We set our window title
  [(NSPanel *)[self window] setFloatingPanel: YES];
  [[self window] setFrameAutosaveName: @"TagEditorWindow"];
  [[self window] setFrameUsingName: @"TagEditorWindow"];

  [writeButton setEnabled: NO];

  [[NSNotificationCenter defaultCenter] addObserver: self
                         selector: @selector(setSongInfo:)
                         name: @"MBSelectedSongInfoChanged"
                                             object: nil];
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [super dealloc];
}

// action methods

- (void) okAction: (id) sender
{  
  [self applyAction: sender];
  [self close];
}

- (void) cancelAction: (id) sender
{
  [self close];
}

- (void) applyAction: (id) sender
{
  [info setSong: [songField stringValue]];
  [info setArtist: [artistField stringValue]];
  [info setAlbum: [albumField stringValue]];

  if (([writeButton state]) && 
      ([[[[info filename] pathExtension] lowercaseString] isEqualToString: @"ogg"]))
    {
      OGGEditor *editor = [[OGGEditor alloc] initWithFile: [info filename]];
      [editor setTitle: [songField stringValue]];
      [editor setArtist: [artistField stringValue]];
      [editor setAlbum: [albumField stringValue]];
      [editor writeToFile: [info filename]];
      RELEASE(editor);
      NSLog(@"editor released");
    }

  [[NSNotificationCenter defaultCenter]
           postNotificationName: @"MBShouldReloadTableViewData"
           object: nil];
  NSLog(@"apply finished");
}

@end
